# coding=UTF-8
class GateWayServerConfig:
    MaxPortNum = 65536
    THREAD_COUNT = 4
    IP = "112.4.154.165"
    PORT = 8262
    CERTFOLDER = "cert"
    ISTOKEN = False
    BACK_LIST = []
    backup1 = {"IP": "218.94.125.135", "Port": 8262}
    BACK_LIST.append(backup1)
    backup2 = {"IP": "221.6.24.39", "Port": 8262}
    BACK_LIST.append(backup2)

    # UAT的gateway地址
    UAT_IP = "168.63.66.183"
    UAT_PORT = 9242
    UAT_BACK_LIST = []
    uatbackup1 = {"IP": "168.63.70.228", "Port": 9242}
    UAT_BACK_LIST.append(uatbackup1)

    # SIT的gateway地址
    SIT_IP = "168.61.2.39"
    SIT_PORT = 9072
    SIT_BACK_LIST = []
    sitbackup1 = {"IP": "168.63.17.150", "Port": 9072}
    SIT_BACK_LIST.append(sitbackup1)


class QueryServerConfig:
    MAX_QUERY_SIZE = 104857600
    QUERY_ADDRESS = "service-insight.htsc.com.cn:9552"
    QUERY_CERT = "cert/HTInsightCA.crt"
    QUERY_IS_SSL = True

    # UAT的query地址
    UAT_QUERY_IS_SSL = True
    UAT_QUERY_ADDRESS = "insight-uat01.htsc.com.cn:9463"
    UAT_QUERY_CERT = "cert/HTISCA.crt"

    # SIT的query地址
    SIT_QUERY_IS_SSL = False
    SIT_QUERY_ADDRESS = "168.61.2.39:9163"
    SIT_QUERY_CERT = "cert/HTISCA.crt"


class Play_Back_Type:
    Play_Back = 0
    Play_Back_Oneday = 1


class QueryType:
    QUERY_CONSTANT = 1  # 查询历史上所有的指定证券的基础
    QUERY_CONSTANT_TODAY = 2  # 查询今日最新的指定证券的基础信息
    QUERY_ETFBASICINFO = 4  # 查询指定ETF证券的基础信息
    QUERY_TICK = 3  # 查询指定证券的最新快照信息


class SortType:
    Sort_MDTime = 0  # 0：默认方式，按照MDTime排序
    Sort_RecivedTime = 1  # 1：按照RecivedTime排序


class MDPlaybackExrightsType:
    DEFAULT_EXRIGHTS_TYPE = 0  # 默认
    NO_EXRIGHTS = 1  # 不复权
    FORWARD_EXRIGHTS = 2  # 向前复权
    BACKWARD_EXRIGHTS = 3  # 向后复权


class EMarketDataType:
    UNKNOWN_DATA_TYPE = 0
    MD_TICK = 1
    MD_TRANSACTION = 2
    MD_ORDER = 3
    MD_CONSTANT = 4
    DYNAMIC_PACKET = 5
    MD_ETF_BASICINFO = 6
    MD_KLINE_1MIN = 20
    MD_KLINE_5MIN = 21
    MD_KLINE_15MIN = 22
    MD_KLINE_30MIN = 23
    MD_KLINE_60MIN = 24
    MD_KLINE_1D = 25
    MD_KLINE_15S = 26
    MD_TWAP_1MIN = 30
    MD_TWAP_1S = 31
    MD_VWAP_1MIN = 40
    MD_VWAP_1S = 41
    MD_SIMPLE_TICK = 50
    AD_UPSDOWNS_ANALYSIS = 51
    AD_INDICATORS_RANKING = 52
    AD_VOLUME_BYPRICE = 53
    AD_FUND_FLOW_ANALYSIS = 54
    AD_ORDERBOOK_SNAPSHOT = 55
    AD_ORDERBOOK_SNAPSHOT_WITH_TICK = 56
    AD_CHIP_DISTRIBUTION = 57
    MD_WARRANT = 58
    MD_SECURITY_LENDING = 59
    AD_NEWS = 60
    AD_STARING_RESULT = 61
    AD_DERIVED_ANALYSIS = 62
    MD_FI_QUOTE = 70
    MD_QUOTE = 71
    MD_QB_QUOTE = 72
    MD_QB_TRANSACTION = 73
    MD_SL_ORDER = 74
    MD_SL_TRANSACTION = 75
    MD_USA_ORDER = 76
    MD_USA_TRANSACTION = 77
    REPLAY_MD_TICK_WITH_TRANSACTION = 101
    REPLAY_MD_TICK_WITH_ORDER = 102
    REPLAY_MD_TICK_WITH_TRANSACTION_AND_ORDER = 103
    REPLAY_MD_TICK = 104
    REPLAY_MD_TRANSACTION = 105
    REPLAY_MD_ORDER = 106
    REPLAY_MD_TRANSACTION_AND_ORDER = 107


class ESubscribeActionType:
    COVERAGE = 1
    ADD = 2
    DECREASE = 3
    CANCEL = 4


class ESecurityType:
    DefaultSecurityType = 0
    IndexType = 1
    StockType = 2
    FundType = 3
    BondType = 4
    RepoType = 5
    WarrantType = 6
    OptionType = 7
    FuturesType = 8
    ForexType = 9
    RateType = 10
    NmetalType = 11
    CashBondType = 12
    SpotType = 13
    InsightType = 20
    OtherType = 99


class ESecurityIDSource:
    DefaultSecurityIDSource = 0
    XSHG = 101
    XSHE = 102
    NEEQ = 103
    XSHGFI = 104
    XSHECA = 105
    XHKG = 203
    HKSC = 204
    HGHQ = 205
    CCFX = 301
    XSGE = 302
    INE = 303
    SGEX = 401
    XCFE = 501
    CCDC = 502
    XDCE = 601
    XZCE = 602
    SWS = 701
    CNI = 702
    CSI = 703
    HTIS = 801
    MORN = 802
    QB = 803
    SPDB = 804
    HTSM = 805
    SCB = 806
    LSE = 901
    LME = 902
    LIFFE = 903
    ICEU = 904
    BSE = 905
    NSE = 906
    NEX = 907
    APEX = 908
    ICE_SG = 909
    SGX = 910
    TSE = 911
    TOCOM = 912
    OSE = 913
    EUREX = 914
    ICE = 915
    CME = 916
    CBOT = 917
    CBOE = 918
    AMEX = 919
    US = 920
    NYSE = 921
    NYMEX = 922
    COMEX = 923
    ICUS = 924
    NASDAQ = 925
    BBG = 926
    BMD = 927
    LUXSE = 928
    KRX = 929
    MICEX = 930
    ASE = 931
    ISE = 932
    DME = 933
    IHK = 934
    STOXX = 935
    SPI = 936
    NIKKEI = 937
    DJI = 938
    BATS = 939
    IEX = 940


class EPlaybackTaskStatus:
    DEFAULT_EXRIGHTS_TYPE = 0
    NO_EXRIGHTS = 10
    FORWARD_EXRIGHTS = 11
    BACKWARD_EXRIGHTS = 12
    DEFAULT_CONTROL_TYPE = 0
    CANCEL_TASK = 1
    SET_PLAYBACK_RATE = 2
    DEFAULT_STATUS = 0
    INITIALIZING = 11
    PREPARING = 12
    PREPARED = 13
    RUNNING = 14
    APPENDING = 15
    CANCELED = 16
    COMPLETED = 17
    FAILED = 18


class Error_Type:
    CONNECT_TO_SERVER_FAILED = -1000  # 连接服务器失败，网络不通
    SERVER_REJECT_LOGIN = -1001  # 服务器拒绝登录
    INVALID_MESSAGE_HANDLE = -1002  # 无效的消息处理对象，需要在登录前设置
    STREAM_SEND_ERROR = -1003  # 链路故障，发送报文失败
    STREAM_RECV_ERROR = -1004  # 链路故障，接收报文失败
    LOGIN_WAIT_TIMEOUT = -1005  # 登录超时
    INVALID_LOGIN_RESPONSE_MESSAGE = -1006  # 无效的登录应答消息，内部错误
    LOGIN_ALL_SERVERS_FAILED = -1007  # 所有服务器登录失败
    SERVICE_DISCOVERY_RESPONSE_INVALID = -1008  # 无效的服务发现应答消息
    SERVICE_DISCOVERY_RESPONSE_FAILURE = -1009  # 服务发现应答结果为：失败
    VALID_SERVER_NOT_EXIST = -1010  # 不存在可登录的服务器
    STREAM_INVALID = -1011  # 链接无效，正在重连
    SEND_WAIT_TIMEOUT_ERROR = -1012  # 请求发送超时
    RECEIVE_WAIT_TIMEOUT_ERROR = -1013  # 接收超时

    INVALID_INPUT_PORT = -1020  # 无效的输入端口号，输入检查时报告
    INVALID_INPUT_USER = -1021  # 无效的输入用户名，输入检查检测时报告
    INVALID_INPUT_IP = -1022  # 无效的ip，输入检查检测时报告
    INVALID_CLIENT = -1023  # 无效的客户端

    START_MAINTAIN_THREAD_FAILED = -2100  # 启动链接对象维护变成失败
    START_HEARTBEAT_THREAD_FAILED = -2101  # 启动心跳线程失败
    START_MESSAGE_THREAD_FAILED = -2102  # 启动消息处理线程失败
    START_HANDLE_THREAD_FAILED = -2103  # 启动handle处理线程失败
    STOP_LOGIN_FOR_QUIT = -2104  # 客户端退出，停止登录
    ACQUIRE_QUIT_MUTEX_FAILED = -2105  # 请求锁失败，内部错误
    GET_MESSAGE_FROM_QUEUE_TIMEOUT = -2106  # 获取消息超时
    SUBSCRIBE_RESPONSE_REJECT = -2107  # 订阅应答结果为拒绝
    SUBSCRIBE_RESPONSE_HEADER_ID_NOT_EQUAL = -2108  # 订阅应答消息头不一致，错位的订阅应答，丢弃
    INVALID_EMDC_MESSAGE_TYPE = -2109  # 无效的EMDC消息类型
    INVALID_PACKAGE_SIZE = -2110  # 无效的包长度
    NULL_MESSAGE_POINTER = -2111  # 消息指针为NULL，内部异常
    INVALID_INSIGHT_MESSAGE_HEADER = -2112  # 无效的消息头
    INVALID_INSIGHT_MESSAGE_BODY = -2113  # 无效的消息体
    SERIALIZE_MESSAGE_HEADER_TO_ARRAY_FAILED = -2114  # 序列化消息头失败
    SERIALIZE_MESSAGE_BODY_TO_ARRAY_FAILED = -2115  # 序列化消息体失败
    PARSE_MESSAGE_HEADER_FROM_ARRAY_FAILED = -2116  # 从数组中解析消息头失败
    PARSE_MESSAGE_BODY_FROM_ARRAY_FAILED = -2117  # 从数组中解析消息体失败
    INVALID_INSIGHT_MESSAGE_BUF = -2118  # 无效的消息缓冲
    OUT_OF_MEMORY = -2120  # 内存不足
    INVALID_SUBSCRIBE_INPUT = -2121  # 无效的订阅输入
    SUBSCRIBE_FAILED = -2122  # 订阅失败
    CLIENT_IS_ALREADY_LOGIN = -2300  # 已登录不能重复登录
    WAIT_SUBSCRIBE_RESPONSE_TIMEOUT = -2301  # 等待订阅应答超时
    INVALID_MDQUERY_RESPONSE_MESSAGE = -2302  # 无效的查询应答消息
    WAIT_PLAYBACK_RESPONSE_TIMEOUT = -2303  # 等待回放应答超时
    WAIT_MDQUERY_RESPONSE_TIMEOUT = -2304  # 等待查询应答失败
    PLAYBACK_RESPONSE_REJECT = -2305  # 回放应答结果为：拒绝
    MDQUERY_RESPONSE_ERROR = -2306  # 查询应答错误
    MDQUERY_RESPONSE_FAILURE = -2307  # 查询结果为失败
    ACQUIRE_SUBSCRIBE_MUTEX_FAILED = -2308  # 请求订阅锁失败，内部错误
    INVALID_SUBSCRIBE_MESSAGE_BODY = -2309  # 订阅消息体无效

    INIT_SSL_CONTEXT_FAILED = -2400  # 初始化SSL环境错误
    SSL_VERIFY_PRIVATE_KEY_FAILED = -2401  # SSL验证key失败
    PLAYBACK_CONTROL_RESPONSE_REJECT = -2402  # 回放控制应答结果为：拒绝
    QUERY_RESPONSE_CITATION_REJECT = -2403  # 选择回调函数处理查询回复，却使用引用返回回复
    QUERY_RESPONSE_CALLBACK_REJECT = -2404  # 未选择回调函数处理查询回复，却未使用应用返回回复

    UNKNOWN_PROPERTY_NAME = -2501  # 未知的参数名


class ErrorInfo:
    @staticmethod
    def get_error_code_value(code):
        value = "未知错误类型"
        if code == Error_Type.CONNECT_TO_SERVER_FAILED:
            value = "连接服务器失败，网络不通"
        if code == Error_Type.SERVER_REJECT_LOGIN:
            value = "服务器拒绝登录"
        if code == Error_Type.INVALID_MESSAGE_HANDLE:
            value = "无效的消息处理对象，需要在登录前设置"
        if code == Error_Type.STREAM_SEND_ERROR:
            value = "链路故障，发送报文失败"
        if code == Error_Type.STREAM_RECV_ERROR:
            value = "链路故障，接收报文失败"
        if code == Error_Type.LOGIN_WAIT_TIMEOUT:
            value = "登录超时"
        if code == Error_Type.INVALID_LOGIN_RESPONSE_MESSAGE:
            value = "无效的登录应答消息，内部错误"
        if code == Error_Type.LOGIN_ALL_SERVERS_FAILED:
            value = "所有服务器登录失败"
        if code == Error_Type.SERVICE_DISCOVERY_RESPONSE_INVALID:
            value = "无效的服务发现应答消息"
        if code == Error_Type.SERVICE_DISCOVERY_RESPONSE_FAILURE:
            value = "服务发现应答结果为：失败"
        if code == Error_Type.VALID_SERVER_NOT_EXIST:
            value = "不存在可登录的服务器"
        if code == Error_Type.STREAM_INVALID:
            value = "链接无效，正在重连"
        if code == Error_Type.SEND_WAIT_TIMEOUT_ERROR:
            value = "请求发送超时"
        if code == Error_Type.RECEIVE_WAIT_TIMEOUT_ERROR:
            value = "接收超时"
        if code == Error_Type.INVALID_INPUT_PORT:
            value = "无效的输入端口号，输入检查时报告"
        if code == Error_Type.INVALID_INPUT_USER:
            value = "无效的输入用户名，输入检查检测时报告"
        if code == Error_Type.INVALID_INPUT_IP:
            value = "无效的ip，输入检查检测时报告"
        if code == Error_Type.START_MAINTAIN_THREAD_FAILED:
            value = "启动链接对象维护变成失败"
        if code == Error_Type.START_HEARTBEAT_THREAD_FAILED:
            value = "启动心跳线程失败"
        if code == Error_Type.START_MESSAGE_THREAD_FAILED:
            value = "启动消息处理线程失败"
        if code == Error_Type.START_HANDLE_THREAD_FAILED:
            value = "启动handle处理线程失败"
        if code == Error_Type.STOP_LOGIN_FOR_QUIT:
            value = "客户端退出，停止登录"
        if code == Error_Type.ACQUIRE_QUIT_MUTEX_FAILED:
            value = "请求锁失败，内部错误"
        if code == Error_Type.GET_MESSAGE_FROM_QUEUE_TIMEOUT:
            value = "获取消息超时"
        if code == Error_Type.SUBSCRIBE_RESPONSE_REJECT:
            value = "订阅应答结果为拒绝"
        if code == Error_Type.SUBSCRIBE_RESPONSE_HEADER_ID_NOT_EQUAL:
            value = "订阅应答消息头不一致，错位的订阅应答，丢弃"
        if code == Error_Type.INVALID_EMDC_MESSAGE_TYPE:
            value = "无效的EMDC消息类型"
        if code == Error_Type.INVALID_PACKAGE_SIZE:
            value = "无效的包长度"
        if code == Error_Type.NULL_MESSAGE_POINTER:
            value = "消息指针为NULL，内部异常"
        if code == Error_Type.INVALID_INSIGHT_MESSAGE_HEADER:
            value = "无效的消息头"
        if code == Error_Type.INVALID_INSIGHT_MESSAGE_BODY:
            value = "无效的消息体 "
        if code == Error_Type.SERIALIZE_MESSAGE_HEADER_TO_ARRAY_FAILED:
            value = "序列化消息头失败"
        if code == Error_Type.SERIALIZE_MESSAGE_BODY_TO_ARRAY_FAILED:
            value = "序列化消息体失败"
        if code == Error_Type.PARSE_MESSAGE_HEADER_FROM_ARRAY_FAILED:
            value = "从数组中解析消息头失败"
        if code == Error_Type.PARSE_MESSAGE_BODY_FROM_ARRAY_FAILED:
            value = "从数组中解析消息体失败"
        if code == Error_Type.INVALID_INSIGHT_MESSAGE_BUF:
            value = "无效的消息缓冲"
        if code == Error_Type.OUT_OF_MEMORY:
            value = "内存不足"
        if code == Error_Type.INVALID_SUBSCRIBE_INPUT:
            value = "无效的订阅输入"
        if code == Error_Type.SUBSCRIBE_FAILED:
            value = "订阅失败"
        if code == Error_Type.CLIENT_IS_ALREADY_LOGIN:
            value = "已登录不能重复登录"
        if code == Error_Type.WAIT_SUBSCRIBE_RESPONSE_TIMEOUT:
            value = "等待订阅应答超时"
        if code == Error_Type.INVALID_MDQUERY_RESPONSE_MESSAGE:
            value = "无效的查询应答消息"
        if code == Error_Type.WAIT_PLAYBACK_RESPONSE_TIMEOUT:
            value = "等待回放应答超时"
        if code == Error_Type.WAIT_MDQUERY_RESPONSE_TIMEOUT:
            value = "等待查询应答失败"
        if code == Error_Type.PLAYBACK_RESPONSE_REJECT:
            value = "回放应答结果为：拒绝"
        if code == Error_Type.MDQUERY_RESPONSE_ERROR:
            value = "查询应答错误"
        if code == Error_Type.MDQUERY_RESPONSE_FAILURE:
            value = "查询结果为失败"
        if code == Error_Type.ACQUIRE_SUBSCRIBE_MUTEX_FAILED:
            value = "请求订阅锁失败，内部错误"
        if code == Error_Type.INVALID_SUBSCRIBE_MESSAGE_BODY:
            value = "订阅消息体无效"
        if code == Error_Type.INIT_SSL_CONTEXT_FAILED:
            value = "初始化SSL环境错误"
        if code == Error_Type.SSL_VERIFY_PRIVATE_KEY_FAILED:
            value = "SSL验证key失败"
        if code == Error_Type.PLAYBACK_CONTROL_RESPONSE_REJECT:
            value = "回放控制应答结果为：拒绝"
        if code == Error_Type.QUERY_RESPONSE_CITATION_REJECT:
            value = "选择回调函数处理查询回复，却使用引用返回回复"
        if code == Error_Type.QUERY_RESPONSE_CALLBACK_REJECT:
            value = "未选择回调函数处理查询回复，却未使用应用返回回复"
        if code == Error_Type.UNKNOWN_PROPERTY_NAME:
            value = "未知的参数名"
        else:
            value = "未知错误类型"
        return value



#!/usr/bin/python3
import struct
from sys import platform as _sys_info
try:
    size = struct.calcsize('P')
except struct.error:# Older installations can only query longs
    size = struct.calcsize('l')
if _sys_info.__contains__("win32") or _sys_info.__contains__("Win32"):
    from sys import winver as _version_info
    print(_sys_info)
    print("python version :"+_version_info)
    if size == 8: # 64bits system
        if _version_info.__contains__("3.6"):
            from ..libs.python36.x64 import mdc_gateway_client as mdc_gateway_client
        elif _version_info.__contains__("3.7"):
            from ..libs.python37.x64 import mdc_gateway_client as mdc_gateway_client
        elif _version_info.__contains__("3.8"):
            from ..libs.python38.x64 import mdc_gateway_client as mdc_gateway_client
        elif _version_info.__contains__("3.9"):
            from ..libs.python39.x64 import mdc_gateway_client as mdc_gateway_client
        else:
            raise RuntimeError('Python 3.7 or Python 3.6 required')
    elif size == 4: # 32bits system
        if _version_info.__contains__("3.6"):
            from ..libs.python36.x86 import mdc_gateway_client as mdc_gateway_client
        elif _version_info.__contains__("3.7"):
            from ..libs.python37.x86 import mdc_gateway_client as mdc_gateway_client
        else:
            raise RuntimeError('Python 3.7 or Python 3.6 required')
    else:
        raise RuntimeError('Python 3.7 or Python 3.6 required')
else:
    from com.libs.linux import mdc_gateway_client as mdc_gateway_client


#!/usr/bin/python3
# -*- coding: utf-8 -*-
import json
from ..interface.mdc_gateway_head import mdc_gateway_client

global callback
callback = None

global logs
logs = None


def PyNotify_Regist(notify):
    global callback
    callback = notify


def PyLog_Regist(log):
    global logs
    logs = log
    mdc_gateway_client.Log_Regist(logs)


class PyNotify(mdc_gateway_client.Notify):
    def __init__(self):
        mdc_gateway_client.Notify.__init__(self)
        self.code = b'?$&@'

    def OnMarketData(self, buf, size):
        try:
            result = mdc_gateway_client.cdata(buf, size)

            if callback is not None and (len(result) > 0):
                callback.OnMarketData(json.loads(result))
        except BaseException as e:
            print("Notify OnMarketData error happened!")
            print(buf)
            print(e)

    def OnPlaybackPayload(self, buf, size):
        try:
            result = mdc_gateway_client.cdata(buf, size)
            if callback is not None and (len(result) > 0):
                callback.OnPlaybackPayload(result)
        except BaseException as e:
            print("Notify error happened in OnPlaybackPayload")
            print(buf)
            print(e)

    def OnPlaybackStatus(self, buf, size):
        try:
            result = mdc_gateway_client.cdata(buf, size)
            if callback is not None and (len(result) > 0):
                callback.OnPlaybackStatus(result)

        except BaseException as e:
            print("Notify error happened in OnPlaybackStatus")
            print(buf)
            print(e)

    def OnPlaybackResponse(self, buf, size):
        try:
            result = mdc_gateway_client.cdata(buf, size)
            if callback is not None and (len(result) > 0):
                callback.OnPlaybackResponse(result)
        except BaseException as e:
            print("Notify error happened in OnPlaybackResponse")
            print(buf)
            print(e)

    def OnPlaybackControlResponse(self, buf, size):
        try:
            result = mdc_gateway_client.cdata(buf, size)
            if callback is not None and (len(result) > 0):
                callback.OnPlaybackControlResponse(result)
        except BaseException as e:
            print("Notify error happened in OnPlaybackControlResponse")
            print(buf)
            print(e)

    def OnServiceMessage(self, buf, size):
        try:
            result = mdc_gateway_client.cdata(buf, size)
            if callback is not None and (len(result) > 0):
                callback.OnServiceMessage(result)
        except BaseException as e:
            print("Notify error happened in OnServiceMessage")
            print(buf)
            print(e)

    def OnSubscribeResponse(self, buf, size):
        try:
            result = mdc_gateway_client.cdata(buf, size)
            if callback is not None and (len(result) > 0):
                callback.OnSubscribeResponse(result)
        except BaseException as e:
            print("Notify error happened in OnSubscribeResponse")
            print(buf)
            print(e)

    def OnQueryResponse(self, buf, size):
        try:
            result = mdc_gateway_client.cdata(buf, size)
            if callback is not None and (len(result) > 0):
                callback.OnQueryResponse(result)
        except BaseException as e:
            print("Notify error happened in OnQueryResponse")
            print(buf)
            print(e)

    def OnFinInfoQueryResponse(self, buf, size):
        try:
            result = mdc_gateway_client.cdata(buf, size)
            if callback is not None and (len(result) > 0):
                callback.OnFinInfoQueryResponse(result)
        except BaseException as e:
            print("Notify error happened in OnFinInfoQueryResponse")
            print(buf)
            print(e)

    def OnGeneralError(self, buf, size):
        try:
            result = mdc_gateway_client.cdata(buf, size)
            if callback is not None and (len(result) > 0):
                callback.OnGeneralError(result)
        except BaseException as e:
            print("Notify error happened in OnGeneralError")
            print(buf)
            print(e)

    def OnLoginSuccess(self):
        if callback is not None:
            callback.OnLoginSuccess()

    def OnLoginFailed(self, error_no, message):
        try:
            if callback is not None:
                callback.OnLoginFailed(error_no, message)
        except BaseException as e:
            print("Notify error happened in OnLoginFailed")
            print(e)

    def OnNoConnections(self):
        if callback is not None:
            callback.OnNoConnections()

    def OnReconnect(self):
        if callback is not None:
            callback.OnReconnect()



from ..interface.mdc_gateway_base_define import EMarketDataType


class market_service(object):
    def __init__(self):
       pass
    # ************************************处理数据订阅************************************
    # 处理订阅的股票Tick数据，mdStock格式为json格式
    # 订阅的证券类型为ESecurityType.StockType
    def onSubscribe_StockType_MD_TICK(self, mdStock):
        pass

    # 处理订阅的指数Tick数据，mdIndex格式为json格式
    # 订阅的证券类型为ESecurityType.IndexType
    def onSubscribe_IndexType_MD_TICK(self, mdIndex):
        pass
        # print(mdIndex)

    # 处理订阅的债券Tick数据，mdBond格式为json格式
    # 订阅的证券类型为ESecurityType.BondType
    def onSubscribe_BondType_MD_TICK(self, mdBond):
        pass
        # print(mdBond)

    # 处理订阅的基金Tick数据，mdFund格式为json格式
    # 订阅的证券类型为ESecurityType.FundType
    def onSubscribe_FundType_MD_TICK(self, mdFund):
        pass
        # print(mdFund)

    # 处理订阅的期权Tick数据，mdOption格式为json格式
    # 订阅的证券类型为ESecurityType.OptionType
    def onSubscribe_OptionType_MD_TICK(self, mdOption):
        pass
        # print(mdOption)

    # 处理订阅的期货Tick数据，mdFuture格式为json格式
    # 订阅的证券类型为ESecurityType.OptionType
    def onSubscribe_FuturesType_MD_TICK(self, mdFuture):
        pass
        # print(mdFuture)

    # 处理订阅的逐笔成交，marketdatajson格式为json格式
    # 订阅的证券类型为ESecurityType.MD_TRANSACTION
    # 处理订阅的逐笔委托，格式为json格式
    # 订阅的证券类型为ESecurityType.MD_ORDER
    def onSubscribe_MD_TRANSACTION_and_MD_ORDER(self, marketdatajson):
        if marketdatajson["marketDataType"] == EMarketDataType.MD_TRANSACTION:  # 逐笔成交
            mdtransaction = marketdatajson["mdTransaction"]
            pass
            # print(mdtransaction)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_ORDER:  # 逐笔委托
            mdorder = marketdatajson["mdOrder"]
            pass
            # print(mdorder)
        # print(marketdatajsons)

    # 处理订阅的K线指标模型，marketdatajson格式为json格式
    # 订阅的数据类型为EMarketDataType.MD_KLINE_15S 返回#15秒钟K线
    # 订阅的数据类型为EMarketDataType.MD_KLINE_1MIN 返回#1分钟K线
    # 订阅的数据类型为EMarketDataType.MD_KLINE_5MIN 返回#5分钟K线
    # 订阅的数据类型为EMarketDataType.MD_KLINE_15MIN 返回#15分钟K线
    # 订阅的数据类型为EMarketDataType.MD_KLINE_30MIN 返回#30分钟K线
    # 订阅的数据类型为EMarketDataType.MD_KLINE_60MIN 返回#60分钟K线
    # 订阅的数据类型为EMarketDataType.MD_KLINE_1D 返回#日K线
    def onSubscribe_MD_KLINE(self, marketdatajson):
        if marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_15S:  # 15秒钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_1MIN:  # 1分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_5MIN:  # 5分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_15MIN:  # 15分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_30MIN:  # 30分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_60MIN:  # 60分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_1D:  # 日K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)

        # print(marketdatajson)

    # 处理订阅的资金流向数据，mdFundFlowAnalysis格式为json格式
    # 订阅的证券类型为ESecurityType.AD_FUND_FLOW_ANALYSIS
    def onSubscribe_AD_FUND_FLOW_ANALYSIS(self, mdFundFlowAnalysis):
        pass
        # print(mdFundFlowAnalysis)

    # 处理订阅的融券通数据，mdSecurityLending格式为json格式
    # 订阅的证券类型为ESecurityType.MD_SECURITY_LENDING
    def onSubscribe_MD_SECURITY_LENDING(self, mdSecurityLending):
        pass
        # print(mdSecurityLending)

    # ************************************处理回放数据************************************
    # 处理回放的股票Tick数据，mdStock格式为json格式
    # 回放的证券类型为ESecurityType.StockType
    def onPlayback_StockType_MD_TICK(self, mdStock):
        pass
        # print(mdStock)

    # 处理回放的指数Tick数据，mdIndex格式为json格式
    # 回放的证券类型为ESecurityType.IndexType
    def onPlayback_IndexType_MD_TICK(self, mdIndex):
        pass
        # print(mdIndex)

    # 处理回放的债券Tick数据，mdBond格式为json格式
    # 回放的证券类型为ESecurityType.BondType
    def onPlayback_BondType_MD_TICK(self, mdBond):
        pass
        # print(mdBond)

    # 处理回放的基金Tick数据，mdFund格式为json格式
    # 回放的证券类型为ESecurityType.FundType
    def onPlayback_FundType_MD_TICK(self, mdFund):
        pass
        # print(mdFund)

    # 处理回放的期权Tick数据，mdOption格式为json格式
    # 回放的证券类型为ESecurityType.OptionType
    def onPlayback_OptionType_MD_TICK(self, mdOption):
        pass
        # print(mdOption)

    # 处理回放的期货Tick数据，mdFuture格式为json格式
    # 回放的证券类型为ESecurityType.OptionType
    def onPlayback_FuturesType_MD_TICK(self, mdFuture):
        pass
        # print(mdFuture)

    # 处理回放的逐笔成交，marketdatajson格式为json格式
    # 回放的证券类型为ESecurityType.MD_TRANSACTION
    # 处理回放的逐笔委托，格式为json格式
    # 回放的证券类型为ESecurityType.MD_ORDER
    def onPlayback_MD_TRANSACTION_and_MD_ORDER(self, marketdatajson):
        if marketdatajson["marketDataType"] == EMarketDataType.MD_TRANSACTION:  # 逐笔成交
            mdtransaction = marketdatajson["mdTransaction"]
            pass
            # print(mdtransaction)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_ORDER:  # 逐笔委托
            mdorder = marketdatajson["mdOrder"]
            pass
            # print(mdorder)
        # print(marketdatajsons)

    # 处理回放的K线指标模型，marketdatajson格式为json格式
    # 回放的数据类型为EMarketDataType.MD_KLINE_15S 返回#15秒钟K线
    # 回放的数据类型为EMarketDataType.MD_KLINE_1MIN 返回#1分钟K线
    # 回放的数据类型为EMarketDataType.MD_KLINE_5MIN 返回#5分钟K线
    # 回放的数据类型为EMarketDataType.MD_KLINE_15MIN 返回#15分钟K线
    # 回放的数据类型为EMarketDataType.MD_KLINE_30MIN 返回#30分钟K线
    # 回放的数据类型为EMarketDataType.MD_KLINE_60MIN 返回#60分钟K线
    # 回放的数据类型为EMarketDataType.MD_KLINE_1D 返回#日K线
    def onPlayback_MD_KLINE(self, marketdatajson):
        if marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_15S:  # 15秒钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_1MIN:  # 1分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_5MIN:  # 5分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_15MIN:  # 15分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_30MIN:  # 30分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_60MIN:  # 60分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_1D:  # 日K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)

        # print(marketdatajson)

    # 处理回放的资金流向数据，mdFundFlowAnalysis格式为json格式
    # 回放的证券类型为ESecurityType.AD_FUND_FLOW_ANALYSIS
    def onPlayback_AD_FUND_FLOW_ANALYSIS(self, mdFundFlowAnalysis):
        pass
        # print(mdFundFlowAnalysis)

    # 处理回放的融券通数据，mdSecurityLending格式为json格式
    # 回放的证券类型为ESecurityType.MD_SECURITY_LENDING
    def onPlayback_MD_SECURITY_LENDING(self, mdSecurityLending):
        pass
        # print(mdSecurityLending)

    # 处理回放的状态，status格式为string格式
    def onPlaybackStatus(self, status):
        pass
        # print(status)

    # 处理回放请求返回结果，response格式为string格式
    def onPlaybackResponse(self, response):
        pass
        # print(response)

    # 处理回放控制请求返回结果，response格式为string格式
    def onPlaybackControlResponse(self, response):
        pass
        # print(response)

    # ************************************处理查询请求返回结果************************************
    # 处理查询历史上所有的指定证券的基础信息 query_mdcontant_type()的返回结果，queryresponse格式为list[json]
    # 处理查询今日最新的指定证券的基础信息 query_last_mdcontant_type()的返回结果，queryresponse格式为list[json]
    # 处理查询历史上所有的指定证券的基础信息 query_mdcontant_id()的返回结果，queryresponse格式为list[json]
    # 处理查询今日最新的指定证券的基础信息 query_last_mdcontant_id()的返回结果，queryresponse格式为list[json]
    # 处理查询指定证券的ETF的基础信息 query_ETFinfo()的返回结果，queryresponse格式为list[json]
    # 处理查询指定证券的最新一条Tick数据 query_last_mdtick()的返回结果，queryresponse格式为list[json]
    def onQueryResponse(self, queryresponse):
        pass
        # for resonse in iter(queryresponse):
        #     # response格式为json格式
        #     print(resonse)




#!/usr/bin/python3
# -*- coding: utf-8 -*-


from com.interface.mdc_gateway_base_define import ESubscribeActionType, ESecurityIDSource, ESecurityType, \
    EMarketDataType, MDPlaybackExrightsType
from com.insight import common, query, playback, subscribe
from com.insight.market_service import market_service


class insightmarketservice(market_service):
    def __init__(self):
        pass

    # ************************************处理数据订阅************************************
    # 处理订阅的股票Tick数据，mdStock格式为json格式
    # 订阅的证券类型为ESecurityType.StockType
    def onSubscribe_StockType_MD_TICK(self, mdStock):
        #print(mdStock)
        pass

    # 处理订阅的指数Tick数据，mdIndex格式为json格式
    # 订阅的证券类型为ESecurityType.IndexType
    def onSubscribe_IndexType_MD_TICK(self, mdIndex):
        pass
        # print(mdIndex)

    # 处理订阅的债券Tick数据，mdBond格式为json格式
    # 订阅的证券类型为ESecurityType.BondType
    def onSubscribe_BondType_MD_TICK(self, mdBond):
        pass
        # print(mdBond)

    # 处理订阅的基金Tick数据，mdFund格式为json格式
    # 订阅的证券类型为ESecurityType.FundType
    def onSubscribe_FundType_MD_TICK(self, mdFund):
        pass
        # print(mdFund)

    # 处理订阅的期权Tick数据，mdOption格式为json格式
    # 订阅的证券类型为ESecurityType.OptionType
    def onSubscribe_OptionType_MD_TICK(self, mdOption):
        pass
        # print(mdOption)

    # 处理订阅的期货Tick数据，mdFuture格式为json格式
    # 订阅的证券类型为ESecurityType.OptionType
    def onSubscribe_FuturesType_MD_TICK(self, mdFuture):
        pass
        # print(mdFuture)

    # 处理订阅的逐笔成交，marketdatajson格式为json格式
    # 订阅的证券类型为ESecurityType.MD_TRANSACTION
    # 处理订阅的逐笔委托，格式为json格式
    # 订阅的证券类型为ESecurityType.MD_ORDER
    def onSubscribe_MD_TRANSACTION_and_MD_ORDER(self, marketdatajson):
        if marketdatajson["marketDataType"] == EMarketDataType.MD_TRANSACTION:  # 逐笔成交
            mdtransaction = marketdatajson["mdTransaction"]
            pass
            # print(mdtransaction)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_ORDER:  # 逐笔委托
            mdorder = marketdatajson["mdOrder"]
            pass
            # print(mdorder)
        # print(marketdatajsons)

    # 处理订阅的K线指标模型，marketdatajson格式为json格式
    # 订阅的数据类型为EMarketDataType.MD_KLINE_15S 返回#15秒钟K线
    # 订阅的数据类型为EMarketDataType.MD_KLINE_1MIN 返回#1分钟K线
    # 订阅的数据类型为EMarketDataType.MD_KLINE_5MIN 返回#5分钟K线
    # 订阅的数据类型为EMarketDataType.MD_KLINE_15MIN 返回#15分钟K线
    # 订阅的数据类型为EMarketDataType.MD_KLINE_30MIN 返回#30分钟K线
    # 订阅的数据类型为EMarketDataType.MD_KLINE_60MIN 返回#60分钟K线
    # 订阅的数据类型为EMarketDataType.MD_KLINE_1D 返回#日K线
    def onSubscribe_MD_KLINE(self, marketdatajson):
        if marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_15S:  # 15秒钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_1MIN:  # 1分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_5MIN:  # 5分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_15MIN:  # 15分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_30MIN:  # 30分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_60MIN:  # 60分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_1D:  # 日K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)

        # print(marketdatajson)

    # 处理订阅的资金流向数据，mdFundFlowAnalysis格式为json格式
    # 订阅的证券类型为ESecurityType.AD_FUND_FLOW_ANALYSIS
    def onSubscribe_AD_FUND_FLOW_ANALYSIS(self, mdFundFlowAnalysis):
        pass
        # print(mdFundFlowAnalysis)

    # 处理订阅的融券通数据，mdSecurityLending格式为json格式
    # 订阅的证券类型为ESecurityType.MD_SECURITY_LENDING
    def onSubscribe_MD_SECURITY_LENDING(self, mdSecurityLending):
        pass
        # print(mdSecurityLending)

    # ************************************处理回放数据************************************
    # 处理回放的股票Tick数据，mdStock格式为json格式
    # 回放的证券类型为ESecurityType.StockType
    def onPlayback_StockType_MD_TICK(self, mdStock):
        pass
        # print(mdStock)

    # 处理回放的指数Tick数据，mdIndex格式为json格式
    # 回放的证券类型为ESecurityType.IndexType
    def onPlayback_IndexType_MD_TICK(self, mdIndex):
        pass
        # print(mdIndex)

    # 处理回放的债券Tick数据，mdBond格式为json格式
    # 回放的证券类型为ESecurityType.BondType
    def onPlayback_BondType_MD_TICK(self, mdBond):
        pass
        # print(mdBond)

    # 处理回放的基金Tick数据，mdFund格式为json格式
    # 回放的证券类型为ESecurityType.FundType
    def onPlayback_FundType_MD_TICK(self, mdFund):
        pass
        # print(mdFund)

    # 处理回放的期权Tick数据，mdOption格式为json格式
    # 回放的证券类型为ESecurityType.OptionType
    def onPlayback_OptionType_MD_TICK(self, mdOption):
        pass
        # print(mdOption)

    # 处理回放的期货Tick数据，mdFuture格式为json格式
    # 回放的证券类型为ESecurityType.OptionType
    def onPlayback_FuturesType_MD_TICK(self, mdFuture):
        pass
        # print(mdFuture)

    # 处理回放的逐笔成交，marketdatajson格式为json格式
    # 回放的证券类型为ESecurityType.MD_TRANSACTION
    # 处理回放的逐笔委托，格式为json格式
    # 回放的证券类型为ESecurityType.MD_ORDER
    def onPlayback_MD_TRANSACTION_and_MD_ORDER(self, marketdatajson):
        if marketdatajson["marketDataType"] == EMarketDataType.MD_TRANSACTION:  # 逐笔成交
            mdtransaction = marketdatajson["mdTransaction"]
            pass
            # print(mdtransaction)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_ORDER:  # 逐笔委托
            mdorder = marketdatajson["mdOrder"]
            pass
            # print(mdorder)
        # print(marketdatajsons)

    # 处理回放的K线指标模型，marketdatajson格式为json格式
    # 回放的数据类型为EMarketDataType.MD_KLINE_15S 返回#15秒钟K线
    # 回放的数据类型为EMarketDataType.MD_KLINE_1MIN 返回#1分钟K线
    # 回放的数据类型为EMarketDataType.MD_KLINE_5MIN 返回#5分钟K线
    # 回放的数据类型为EMarketDataType.MD_KLINE_15MIN 返回#15分钟K线
    # 回放的数据类型为EMarketDataType.MD_KLINE_30MIN 返回#30分钟K线
    # 回放的数据类型为EMarketDataType.MD_KLINE_60MIN 返回#60分钟K线
    # 回放的数据类型为EMarketDataType.MD_KLINE_1D 返回#日K线
    def onPlayback_MD_KLINE(self, marketdatajson):
        if marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_15S:  # 15秒钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_1MIN:  # 1分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_5MIN:  # 5分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_15MIN:  # 15分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_30MIN:  # 30分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_60MIN:  # 60分钟K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)
        elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_1D:  # 日K线
            mdKLine = marketdatajson["mdKLine"]
            pass
            # print(mdKLine)

        # print(marketdatajson)

    # 处理回放的资金流向数据，mdFundFlowAnalysis格式为json格式
    # 回放的证券类型为ESecurityType.AD_FUND_FLOW_ANALYSIS
    def onPlayback_AD_FUND_FLOW_ANALYSIS(self, mdFundFlowAnalysis):
        pass
        # print(mdFundFlowAnalysis)

    # 处理回放的融券通数据，mdSecurityLending格式为json格式
    # 回放的证券类型为ESecurityType.MD_SECURITY_LENDING
    def onPlayback_MD_SECURITY_LENDING(self, mdSecurityLending):
        pass
        # print(mdSecurityLending)

    # 处理回放的状态，status格式为string格式
    def onPlaybackStatus(self, status):
        pass
        # print(status)

    # 处理回放请求返回结果，response格式为string格式
    def onPlaybackResponse(self, response):
        pass
        # print(response)

    # 处理回放控制请求返回结果，response格式为string格式
    def onPlaybackControlResponse(self, response):
        pass
        # print(response)


    # ************************************处理查询请求返回结果************************************
    # 处理查询历史上所有的指定证券的基础信息 query_mdcontant_type()的返回结果，queryresponse格式为list[json]
    # 处理查询今日最新的指定证券的基础信息 query_last_mdcontant_type()的返回结果，queryresponse格式为list[json]
    # 处理查询历史上所有的指定证券的基础信息 query_mdcontant_id()的返回结果，queryresponse格式为list[json]
    # 处理查询今日最新的指定证券的基础信息 query_last_mdcontant_id()的返回结果，queryresponse格式为list[json]
    # 处理查询指定证券的ETF的基础信息 query_ETFinfo()的返回结果，queryresponse格式为list[json]
    # 处理查询指定证券的最新一条Tick数据 query_last_mdtick()的返回结果，queryresponse格式为list[json]
    def onQueryResponse(self, queryresponse):
        pass
        for resonse in iter(queryresponse):
            # response格式为json格式
            print(resonse)


# ************************************用户登录************************************
# 登陆
# user 用户名
# password 密码
def login():
    markets = insightmarketservice()
    # 登陆前 初始化
    user = "username"
    password = "password"
    # result格式为string格式
    result = common.login(markets, user, password)
    print(result)


# 配置日志打开
# open_trace trace日志开关     True为打开日志False关闭日志
# open_file_log  本地file日志开关     True为打开日志False关闭日志
# open_cout_log  控制台日志开关     True为打开日志False关闭日志
def config(open_trace=True, open_file_log=True, open_cout_log=True):
    common.config(open_trace, open_file_log, open_cout_log)


# ************************************数据订阅************************************
# 根据证券数据来源订阅行情数据,由三部分确定行情数据
# 行情源(SecurityIdSource):XSHG(沪市)|XSHE(深市)|...
# 证券类型(SecurityType):BondType(债)|StockType(股)|FundType(基)|IndexType(指)|OptionType(期权)|...
# 数据类型(MarketDataTypes):MD_TICK(快照)|MD_TRANSACTION(逐笔成交)|MD_ORDER(逐笔委托)|...
# 异步接口，返回函数在functional_test.py
# 结果返回 onSubscribe_StockType_MD_TICK 处理订阅的股票Tick数据
# 结果返回 onSubscribe_IndexType_MD_TICK 处理订阅的指数Tick数据
# 结果返回 onSubscribe_BondType_MD_TICK 处理订阅的债券Tick数据
# 结果返回 onSubscribe_FundType_MD_TICK 处理订阅的基金Tick数据
# 结果返回 onSubscribe_OptionType_MD_TICK 处理订阅的期权Tick数据
# 结果返回 onSubscribe_FuturesType_MD_TICK 处理订阅的期货Tick数据
# 结果返回 onSubscribe_MD_TRANSACTION_and_MD_ORDER 处理订阅的逐笔成交和逐笔委托的数据
# 结果返回 onSubscribe_MD_KLINE 处理订阅的K线指标模型数据
# 结果返回 onSubscribe_AD_FUND_FLOW_ANALYSIS 处理订阅的资金流向数据
# 结果返回 onSubscribe_MD_SECURITY_LENDING 处理订阅的融券通数据
def subscribe_by_type():
    # element
    # params1: ESecurityIDSource枚举值 --行情源
    # params2: ESecurityType的枚举值 --证券类型
    # params3: EMarketDataType的枚举值集合 --数据类型
    datatype = ESubscribeActionType.COVERAGE
    marketdatatypes = []
    marketdatatype = {"ESecurityIDSource": ESecurityIDSource.XSHG, "ESecurityType": ESecurityType.StockType,
                      "EMarketDataType": EMarketDataType.MD_SECURITY_LENDING}
    marketdatatypes.append(marketdatatype)
    marketdatatype1 = {"ESecurityIDSource": ESecurityIDSource.XSHG, "ESecurityType": ESecurityType.StockType,
                       "EMarketDataType": EMarketDataType.MD_TICK}
    marketdatatypes.append(marketdatatype1)

    subscribe.subscribe_by_type(marketdatatypes, datatype)


# 根据证券ID来源订阅行情数据
# 异步接口，返回函数在functional_test.py
# 结果返回 onSubscribe_StockType_MD_TICK 处理订阅的股票Tick数据
# 结果返回 onSubscribe_IndexType_MD_TICK 处理订阅的指数Tick数据
# 结果返回 onSubscribe_BondType_MD_TICK 处理订阅的债券Tick数据
# 结果返回 onSubscribe_FundType_MD_TICK 处理订阅的基金Tick数据
# 结果返回 onSubscribe_OptionType_MD_TICK 处理订阅的期权Tick数据
# 结果返回 onSubscribe_FuturesType_MD_TICK 处理订阅的期货Tick数据
# 结果返回 onSubscribe_MD_TRANSACTION_and_MD_ORDER 处理订阅的逐笔成交和逐笔委托的数据
# 结果返回 onSubscribe_MD_KLINE 处理订阅的K线指标模型数据
# 结果返回 onSubscribe_AD_FUND_FLOW_ANALYSIS 处理订阅的资金流向数据
# 结果返回 onSubscribe_MD_SECURITY_LENDING 处理订阅的融券通数据
def subscribe_by_id():
    # params1:HTSCSecurityID华泰内部证券标string --证券代码
    # params2: EMarketDataType的枚举值集合 --数据类型
    datatype = ESubscribeActionType.COVERAGE
    HTSCSecurityIDs = []
    HTSCSecurityID = {"HTSCSecurityID": "002371.SZ", "EMarketDataType": EMarketDataType.MD_TICK}
    HTSCSecurityIDs.append(HTSCSecurityID)

    HTSCSecurityID1 = {"HTSCSecurityID": "002714.SZ", "EMarketDataType": EMarketDataType.MD_TICK}
    HTSCSecurityIDs.append(HTSCSecurityID1)
    subscribe.subscribe_by_id(HTSCSecurityIDs, datatype)


# ************************************数据查询************************************
# 查询findata数据，返回结果list[json]
# 同步接口，返回结果为result，格式为list[json]
def query_fin_info():
    # params1:Hquery_type  int
    # params2:params  dict{}
    query_type = 1102010003
    params = {"HTSC_SECURITY_ID": "510050.SH", "START_DATE": "20210101", "END_DATE": "20210107"}
    result = query.query_fin_info(query_type, params)
    if isinstance(result, list):
        for response in iter(result):
            print(response)
    else:
        print(result)


# 查询历史上所有的指定证券的基础信息
# params:securityIdSource 为市场ESecurityIDSource 枚举值;securityType 为 ESecurityType枚举值
# 异步接口，返回函数在functional_test.py
# 结果返回在onQueryResponse函数
def query_mdcontant_type():
    # params:security_idsource 为 ESecurityIDSource枚举值
    # params:security_type 为 ESecurityType枚举值
    # 沪市 股票
    idsource_and_types = []
    security_idsource = ESecurityIDSource.XSHG
    security_type = ESecurityType.StockType
    idsource_and_type = {"ESecurityIDSource": security_idsource, "ESecurityType": security_type}
    idsource_and_types.append(idsource_and_type)
    security_idsource1 = ESecurityIDSource.XSHE
    security_type1 = ESecurityType.IndexType
    idsource_and_type1 = {"ESecurityIDSource": security_idsource1, "ESecurityType": security_type1}
    idsource_and_types.append(idsource_and_type1)

    query.query_basicInfo_by_type(idsource_and_types, False)


# 查询今日最新的指定证券的基础信息
# params:securityIdSource 为市场ESecurityIDSource 枚举值;securityType 为 ESecurityType枚举值
# 异步接口，返回函数在functional_test.py
# 结果返回在onQueryResponse函数
def query_last_mdcontant_type():
    # 按市场查询
    # 沪市 股票

    idsource_and_types = []
    security_idsource = ESecurityIDSource.XSHG
    security_type = ESecurityType.StockType
    idsource_and_type = {"ESecurityIDSource": security_idsource, "ESecurityType": security_type}
    idsource_and_types.append(idsource_and_type)
    security_idsource1 = ESecurityIDSource.XSHE
    security_type1 = ESecurityType.IndexType
    idsource_and_type1 = {"ESecurityIDSource": security_idsource1, "ESecurityType": security_type1}
    idsource_and_types.append(idsource_and_type1)

    query.query_basicInfo_by_type(idsource_and_types, True)


# 查询历史上所有的指定证券的基础信息 -- 结果返回在onQueryResponse
# param:HTSCSecurityID华泰内部证券标string --证券代码
# params:list[HTSCSecurityID]
# 异步接口，返回函数在functional_test.py
# 结果返回在onQueryResponse函数
def query_mdcontant_id():
    # params:security_idsource 为 ESecurityIDSource枚举值
    # params:security_type 为 ESecurityType枚举值
    # params:security_id_list 为 标的集合
    security_id_list = ["601688.SH", "002714.SZ"]  # 置空表示不额外查询某些标的
    query.query_basicInfo_by_id(security_id_list, False)


# 查询今日最新的指定证券的基础信息 -- 结果返回在onQueryResponse
# param:HTSCSecurityID华泰内部证券标string --证券代码
# params:list[HTSCSecurityID]
# 异步接口，返回函数在functional_test.py
# 结果返回在onQueryResponse函数
def query_last_mdcontant_id():
    # 按市场查询
    # 沪市 股票

    # params:security_id_list 为 标的集合
    security_id_list = ["601688.SH"]  # 置空表示不额外查询某些标的
    query.query_basicInfo_by_id(security_id_list, True)


# 查询指定证券的ETF的基础信息 -- 结果返回在onQueryResponse
# params:securityIdSource 为市场ESecurityIDSource 枚举值;securityType 为 ESecurityType枚举值
# 异步接口，返回函数在functional_test.py
# 结果返回在onQueryResponse函数
def query_ETFinfo():
    # params:securityIDSource 为 ESecurityIDSource枚举值
    # params:securityType 为 ESecurityType枚举值
    query.query_ETFinfo(ESecurityIDSource.XSHG, ESecurityType.FundType)


# 查询指定证券的最新一条Tick数据 --  结果返回在onQueryResponse
# params:securityIdSource 为市场ESecurityIDSource 枚举值;securityType 为 ESecurityType枚举值
# 异步接口，返回函数在functional_test.py
# 结果返回在onQueryResponse函数
def query_last_mdtick():
    # params:security_idsource 为 ESecurityIDSource枚举值
    # params:security_type 为 ESecurityType枚举值
    # 沪市 股票
    query.query_last_tick_by_type(ESecurityIDSource.XSHG, ESecurityType.StockType)


# ************************************回放************************************
# 回放接口 (注意：securitylist 和 securityIdList取并集!!!)
# 回放限制
# 对于回放而言，时间限制由股票只数和天数的乘积决定，要求 回放只数 × 回放天数 × 证券权重 ≤ 450，交易时间段内回放功能 乘积<=200。
# Tick/Transaction/Order回放时间范围限制是30天，每支证券权重为1，即可以回放15只股票30天以内的数据或450支股票1天内数据。
# 日K数据回放时间范围限制是365天，每支证券权重为0.005。
# 分钟K线数据回放时间范围限制是90天，每支证券权重0.05。
# 数据最早可以回放到 2017年1月2日
# 异步接口，返回函数在functional_test.py
# 结果返回 onPlayback_StockType_MD_TICK 处理回放的股票Tick数据
# 结果返回 onPlayback_IndexType_MD_TICK 处理回放的指数Tick数据
# 结果返回 onPlayback_BondType_MD_TICK 处理回放的债券Tick数据
# 结果返回 onPlayback_FundType_MD_TICK 处理回放的基金Tick数据
# 结果返回 onPlayback_OptionType_MD_TICK 处理回放的期权Tick数据
# 结果返回 onPlayback_FuturesType_MD_TICK 处理回放的期货Tick数据
# 结果返回 onPlayback_MD_TRANSACTION_and_MD_ORDER 处理回放的逐笔成交和逐笔委托的数据
# 结果返回 onPlayback_MD_KLINE 处理回放的K线指标模型数据
# 结果返回 onPlayback_AD_FUND_FLOW_ANALYSIS 处理回放的资金流向数据
# 结果返回 onPlayback_MD_SECURITY_LENDING 处理回放的融券通数据
def play_back():
    # 回放数据类型 EMarketDataType 详情请参阅 数据手册EMarketDataType
    # 示例：MD_TICK
    marketdata_type = EMarketDataType.MD_TICK

    # 是否复权 EPlaybackTaskStatus 详情请参阅 数据手册EPlaybackTaskStatus
    # 示例： 不复权
    exrights_type = MDPlaybackExrightsType.DEFAULT_EXRIGHTS_TYPE

    # 回放时间起：start_time 注意格式
    # 回放时间止：stop_time  注意格式
    # 回放时间起止间隔 不超过上述 时间范围限制
    start_time = "20220420090000"
    stop_time = "20220420150000"

    htscsecurityID_and_types = []
    marketdata_type = EMarketDataType.MD_TICK
    htscsecurityID_and_type = {"HTSCSecurityID": "601688.SH", "EMarketDataType": marketdata_type}
    htscsecurityID_and_types.append(htscsecurityID_and_type)
    marketdata_type1 = EMarketDataType.MD_TICK
    htscsecurityID_and_type1 = {"HTSCSecurityID": "000014.SZ", "EMarketDataType": marketdata_type1}
    htscsecurityID_and_types.append(htscsecurityID_and_type1)

    # 特别注意！！！！
    # security_id_list 注意回放限制
    playback.playback(htscsecurityID_and_types, exrights_type, start_time, stop_time)


# 盘中回放接口 --securitylist 和 securityIdList取并集
# Can only query data for one day
# 异步接口，返回函数在functional_test.py
# 结果返回 onPlayback_StockType_MD_TICK 处理回放的股票Tick数据
# 结果返回 onPlayback_IndexType_MD_TICK 处理回放的指数Tick数据
# 结果返回 onPlayback_BondType_MD_TICK 处理回放的债券Tick数据
# 结果返回 onPlayback_FundType_MD_TICK 处理回放的基金Tick数据
# 结果返回 onPlayback_OptionType_MD_TICK 处理回放的期权Tick数据
# 结果返回 onPlayback_FuturesType_MD_TICK 处理回放的期货Tick数据
# 结果返回 onPlayback_MD_TRANSACTION_and_MD_ORDER 处理回放的逐笔成交和逐笔委托的数据
# 结果返回 onPlayback_MD_KLINE 处理回放的K线指标模型数据
# 结果返回 onPlayback_AD_FUND_FLOW_ANALYSIS 处理回放的资金流向数据
# 结果返回 onPlayback_MD_SECURITY_LENDING 处理回放的融券通数据
def play_back_oneday():
    # 回放数据类型 EMarketDataType 详情请参阅 数据手册EMarketDataType
    # 示例：MD_TICK

    # 是否复权 EPlaybackTaskStatus 详情请参阅 数据手册EPlaybackTaskStatus
    # 示例： 不复权
    exrights_type = MDPlaybackExrightsType.DEFAULT_EXRIGHTS_TYPE
    htscsecurityID_and_types = []
    marketdata_type = EMarketDataType.MD_TICK
    htscsecurityID_and_type = {"HTSCSecurityID": "601688.SH", "EMarketDataType": marketdata_type}
    htscsecurityID_and_types.append(htscsecurityID_and_type)
    marketdata_type1 = EMarketDataType.MD_TICK
    htscsecurityID_and_type1 = {"HTSCSecurityID": "000014.SZ", "EMarketDataType": marketdata_type1}
    htscsecurityID_and_types.append(htscsecurityID_and_type1)

    # 特别注意！！！！
    # security_id_list 注意回放限制
    playback.play_back_oneday(htscsecurityID_and_types, exrights_type)


# 获取当前版本号
def get_version():
    print(common.get_version())


# 释放资源
def fini():
    common.fini()


# 使用指导：登陆 -> 订阅/查询/回放 -> 退出
def main(markets=None):

    # 登陆部分调用
    get_version()
    login()
    #config(True, True, True)
    # 订阅部分接口调用

    subscribe_by_type()
    # subscribe_by_id()
    # 查询部分接口调用
    # query_fin_info()
    # query_mdcontant_type()
    # query_last_mdcontant_type()
    # query_mdcontant_id()
    # query_last_mdcontant_id()

    #query_ETFinfo()
    # query_last_mdtick()
    # 回放部分接口调用
    # play_back()
    # play_back_oneday()
    # 退出释放资源
    fini()


if __name__ == "__main__":
    # insight SDK 采用网络异步方式 ---- 请求访问和数据返回 异步交互
    # 这里是 functional_test.py 是 登陆、订阅、查询、回测（也称回放）操作部分
    # 以订阅为例： functional_test.py 中，执行登陆->订阅,操作结束后,数据通过 data_handle.py中 OnRecvMarkertData()的成员方法回调返回（回调详情请参照使用手册）
    main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
import os
import sys
import site
from ..data_handle import get_interface
from .. import data_handle
from ..interface.mdc_gateway_base_define import EMarketDataType, ESecurityIDSource, ESecurityType, \
    ESubscribeActionType, MDPlaybackExrightsType, GateWayServerConfig, QueryServerConfig



# 登陆
# params1: 用户名
# params2: 密码
# params3:
# params2: 密码
def login(marketservice, Username, Password, IP=GateWayServerConfig.IP, Port=GateWayServerConfig.PORT,
          backupIP=GateWayServerConfig.BACK_LIST):
    # 登陆前 初始化
    result = ""
    if not ((type(Port) == int) and 0 < Port < GateWayServerConfig.MaxPortNum):
        result = "Port has invalid format"
        return result
    Initial(marketservice)

    abs_path = os.path.abspath(__file__)
    size = len(abs_path)
    abs_path = abs_path[0:size-17]
    print(f'The current path:{abs_path}')
    ret = get_interface().login(IP, Port, Username, Password, GateWayServerConfig.ISTOKEN,
                                                abs_path+GateWayServerConfig.CERTFOLDER, backupIP,
                                                QueryServerConfig.QUERY_ADDRESS,
                                                abs_path+QueryServerConfig.QUERY_CERT,
                                                QueryServerConfig.QUERY_IS_SSL)


    if ret != 0:
        result = f'(login failed!!! reason:{get_interface().get_error_code_value(ret)})'
        return result
    result = "login success"
    return result


def loginUAT(marketservice, Username, Password, IP=GateWayServerConfig.UAT_IP, Port=GateWayServerConfig.UAT_PORT,
             backupIP=GateWayServerConfig.UAT_BACK_LIST):
    # 登陆前 初始化
    result = ""
    if not ((type(Port) == int) and 0 < Port < GateWayServerConfig.MaxPortNum):
        result = "Port has invalid format"
        return result

    abs_path = os.path.abspath(__file__)
    size = len(abs_path)
    abs_path = abs_path[0:size-17]

    print(f'The current path:{abs_path}')
    Initial(marketservice)
    ret = get_interface().login(IP, Port, Username, Password, GateWayServerConfig.ISTOKEN,
                                                abs_path+GateWayServerConfig.CERTFOLDER, backupIP,
                                                QueryServerConfig.UAT_QUERY_ADDRESS,
                                                abs_path+QueryServerConfig.UAT_QUERY_CERT,
                                                QueryServerConfig.UAT_QUERY_IS_SSL)

    if ret != 0:
        result = f'(login failed!!! reason:{get_interface().get_error_code_value(ret)})'
        return result
    result = "login success"
    return result


def loginSIT(marketservice, Username, Password, IP=GateWayServerConfig.SIT_IP, Port=GateWayServerConfig.SIT_PORT,
             backupIP=GateWayServerConfig.SIT_BACK_LIST):
    # 登陆前 初始化
    result = ""
    if not ((type(Port) == int) and 0 < Port < GateWayServerConfig.MaxPortNum):
        result = "Port has invalid format"
        return result

    abs_path = os.path.abspath(__file__)
    size = len(abs_path)
    abs_path = abs_path[0:size-17]
    Initial(marketservice)
    ret = get_interface().login(IP, Port, Username, Password, GateWayServerConfig.ISTOKEN,
                                                abs_path+GateWayServerConfig.CERTFOLDER, backupIP,
                                                QueryServerConfig.SIT_QUERY_ADDRESS,
                                                abs_path+QueryServerConfig.SIT_QUERY_CERT,
                                                QueryServerConfig.SIT_QUERY_IS_SSL)

    if ret != 0:
        result = f'(login failed!!! reason:{get_interface().get_error_code_value(ret)})'
        return result
    result = "login success"
    return result


# 获取当前版本号
def get_version():
    return get_interface().get_version()


# 释放资源
def release():
    fini()




# 配置
def config(open_trace=True, open_file_log=True, open_cout_log=True):

    if open_file_log:
        get_interface().python_open_file_log()
    else:
        get_interface().python_close_file_log()
    if open_cout_log:
        get_interface().python_open_cout_log()
    else:
        get_interface().python_close_cout_log()
    if open_trace:
        get_interface().python_open_trace()
    else:
        get_interface().python_close_trace()


# 登陆前 初始化 -- 修改ip映射,流量与日志开关设置,回调函数注册,接管系统日志,自我处理日志
def Initial( marketservice):
    # 添加ip映射
    # get_interface().add_ip_map("168.63.17.150", "127.0.0.1")
    # 流量与日志开关设置
    # open_trace trace流量日志开关 # params:open_file_log 本地日志文件开关  # params:open_cout_log 控制台日志开关
    open_trace = False
    open_file_log = False
    open_cout_log = False
    get_interface().init(open_trace, open_file_log, open_cout_log);

    # 注册回调和接管日志
    # 1.注册回调接口，不注册无法接收数据、处理数据，不会回调data_handle
    callback = data_handle.OnRecvMarketData(marketservice)
    get_interface().setCallBack(callback)

    # 2.接管日志
    # 若想关闭系统日志,自我处理日志,打开本方法
    # 打开本方法后,日志在insightlog.py的PyLog类的方法log(self,line)中也会体现,其中 line为日志内容）
    # use_init = True 系统日志以 get_interface().init 配置的方式记录
    # use_init = False 系统不再记录或打印任何日志,日志只有自行处理部分处理


# 释放资源
def fini():
    get_interface().fini()



#!/usr/bin/python3+

import sys, json

from ..interface.mdc_gateway_base_define import ErrorInfo, SortType, Play_Back_Type, QueryType, QueryServerConfig, GateWayServerConfig

sys.path.insert(1, "./interface")
sys.path.insert(2, "./libs")
from ..interface import mdc_recvdata_interface as recvdata

# from log_handle import PyLog
import time
from multiprocessing import Lock
from ..interface.mdc_gateway_head import mdc_gateway_client


global notifys  # 全局变量，不能做临时变量，需要全局使用,注意生存周期
notifys = recvdata.PyNotify()
notifys.thisown = 0


class GatewayInterface():
    ineffective_size = 9999999999
    min_synchronous_data_len = 10
    service_value = 0
    query_exit = False
    reconnect = False
    login_success = False
    no_connections = False
    reconnect_count = 0
    mutex = Lock()
    task_id_status = {}

    def __init__(self):
        self.service_value = 0
        self.query_exit = False
        self.reconnect = False
        self.login_success = False
        self.no_connections = False
        self.reconnect_count = 0
        self.open_file_log = False
        self.open_cout_log = False
        self.open_trace = True

    def get_service_value(self):
        return self.service_value

    def set_service_value(self, i):
        self.service_value = i

    def get_query_exit(self):
        return self.query_exit

    def set_query_exit(self, value):
        self.query_exit = value

    def get_reconnect(self):
        return self.reconnect

    def set_reconnect(self, value):
        self.reconnect = value

    def get_login_flag(self):
        return self.login_success

    def get_no_connections(self):
        return self.no_connections

    def set_no_connections(self, value):
        self.no_connections = value

    def get_reconnect_count(self):
        return self.reconnect_count

    def set_reconnect_count(self, value):
        self.reconnect_count = value

    def get_version(self):
        ver = str(mdc_gateway_client.python_get_version())
        return ver;

    def get_error_code_value(self, code):
        value = ErrorInfo.get_error_code_value(code)

        return value

    def add_ip_map(self, original_ip, map_ip):
        mdc_gateway_client.python_add_ip_map(original_ip.encode(encoding="utf-8"), map_ip.encode(encoding="utf-8"))

    def delete_ip_map(self, original_ip):
        mdc_gateway_client.python_delete_ip_map(original_ip.encode(encoding="utf-8"))

    def add_ip_port(self, original_port, map_port):
        mdc_gateway_client.python_add_port_map(original_port, map_port)

    def delete_ip_map(self, original_port):
        mdc_gateway_client.python_delete_port_map(original_port)

    def is_trace(self):
        return mdc_gateway_client.python_is_trace();

    def is_compress(self):
        return mdc_gateway_client.python_is_compress();

    def is_responseCallback(self):
        return mdc_gateway_client.python_is_response_callback();

    def init(self, open_trace, open_file_log, open_cout_log):
        mdc_gateway_client.python_init_env()
        mdc_gateway_client.python_open_response_callback()
        self.open_trace = open_trace
        self.open_file_log = open_file_log
        self.open_cout_log = open_cout_log

    def python_open_file_log(self):
        self.open_file_log = True
        mdc_gateway_client.python_open_file_log()

    def python_close_file_log(self):
        self.open_file_log = False
        mdc_gateway_client.python_close_file_log()

    def python_open_cout_log(self):
        self.open_cout_log = True
        mdc_gateway_client.python_open_cout_log()

    def python_close_cout_log(self):
        self.open_cout_log = False
        mdc_gateway_client.python_close_cout_log()

    def python_open_trace(self):
        self.open_trace = True
        mdc_gateway_client.python_open_trace()

    def python_close_trace(self):
        self.open_trace = False
        mdc_gateway_client.python_close_trace()

    def __init_log__(self):
        print(f'open_file_log:{self.open_file_log} open_cout_log:{self.open_cout_log} open_trace:{self.open_trace}')
        if self.open_file_log:
            mdc_gateway_client.python_open_file_log()
        else:
            mdc_gateway_client.python_close_file_log()
        if self.open_cout_log:
            mdc_gateway_client.python_open_cout_log()
        else:
            mdc_gateway_client.python_close_cout_log()
        if self.open_trace:
            mdc_gateway_client.python_open_trace()
        else:
            mdc_gateway_client.python_close_trace()

    def open_compress(self, torf):
        if torf:
            mdc_gateway_client.python_open_compress()
        else:
            mdc_gateway_client.python_close_compress()

    def fini(self):
        mdc_gateway_client.python_logout()
        mdc_gateway_client.python_fini_env()

    # 订阅-全订阅 param:marketDataTypeList 为 EMarketDataType 枚举值集合
    def subscribebytype(self, datatype, marketdatatypes):
        if self.get_login_flag():
            subscribejson = {"DataType": datatype}
            subscribejson["MarketdataTypes"] = marketdatatypes
            subscribestr = json.dumps(subscribejson)

            ret = mdc_gateway_client.python_subscribe(subscribestr, len(subscribestr))
            if ret < 0:
                print("subscribeAll failed!!! reason:%s" % (self.get_error_code_value(ret)))
                exit(-1)
        else:
            print("Unsuccessful login")

    def subscribebyid(self, datatype, htscSecurityids):
        if self.get_login_flag():
            subscribejson = {"DataType": datatype}
            subscribejson["HTSCSecurityIDs"] = htscSecurityids
            subscribestr = json.dumps(subscribejson)

            ret = mdc_gateway_client.python_subscribe(subscribestr, len(subscribestr))
            if ret < 0:
                print("subscribeAll failed!!! reason:%s" % (self.get_error_code_value(ret)))
                exit(-1)
        else:
            print("Unsuccessful login")

    # 回放  params:securityIdList 为标的 str值集合;marketDataType 为 EMarketDataType 枚举值集合;exrightsType 为EPlaybackExrightsType枚举值,startTime str类型;stopTime str类型
    def playCallback(self, htscsecurityID_and_types, exrightsType, startTime, stopTime, isMdtime=True):
        if self.get_login_flag():
            for htscsecurityID_and_type in htscsecurityID_and_types:
                playbackjson = {}
                playbackjson["TaskID"] = mdc_gateway_client.python_get_task_id()
                self.mutex.acquire()
                self.task_id_status[playbackjson["TaskID"]] = 0
                self.mutex.release()

                htscSecurityIDs = []

                htscSecurityID = {}
                htscSecurityID["HTSCSecurityID"] = htscsecurityID_and_type["HTSCSecurityID"]
                htscSecurityID["EMarketDataType"] = htscsecurityID_and_type["EMarketDataType"]
                htscSecurityIDs.append(htscSecurityID)

                playbackjson["HTSCSecurityIDs"] = htscSecurityIDs
                playbackjson["ExrightsType"] = exrightsType
                playbackjson["StartTime"] = startTime
                playbackjson["EndTime"] = stopTime

                if isMdtime:
                    playbackjson["Sort"] = SortType.Sort_MDTime
                else:
                    playbackjson["Sort"] = SortType.Sort_RecivedTime

                playbackjson["DataType"] = Play_Back_Type.Play_Back

                playback_info = json.dumps(playbackjson)
                while (self.task_id_status.__len__() >= 5):
                    time.sleep(1)

                ret = mdc_gateway_client.python_request_playback(playback_info, len(playback_info))
                if ret < 0:
                    print(self.get_error_code_value(ret))
                    print("playCallback failed!!!")
                    exit(-1)

            while (self.task_id_status.__len__() > 0):
                time.sleep(1)
        else:
            print("Unsuccessful login")

    # 回放  params:securityIdList 为标的 str值集合;marketDataType 为 EMarketDataType 枚举值集合;exrightsType 为EPlaybackExrightsType枚举值,startTime str类型;stopTime str类型
    def playSortCallback(self, marketDataTypes, startTime, stopTime, exrightsType, sort):
        if self.get_login_flag():
            for marketDataType in marketDataTypes:
                playbackjson = {}
                playbackjson["TaskID"] = mdc_gateway_client.python_get_task_id()
                self.mutex.acquire()
                self.task_id_status[playbackjson["TaskID"]] = 0
                self.mutex.release()

                htscSecurityIDs = []

                htscSecurityID = {}
                htscSecurityID["HTSCSecurityID"] = marketDataType["HTSCSecurityID"]
                htscSecurityID["EMarketDataType"] = marketDataType["EMarketDataType"]
                htscSecurityIDs.append(htscSecurityID)

                playbackjson["HTSCSecurityIDs"] = htscSecurityIDs
                playbackjson["ExrightsType"] = exrightsType
                playbackjson["StartTime"] = startTime
                playbackjson["EndTime"] = stopTime

                playbackjson["Sort"] = sort
                playbackjson["DataType"] = Play_Back_Type.Play_Back

                playback_info = json.dumps(playbackjson)
                while (self.task_id_status.__len__() >= 5):
                    time.sleep(1)

                ret = mdc_gateway_client.python_request_playback(playback_info, len(playback_info))
                if ret < 0:
                    print(self.get_error_code_value(ret))
                    print("playCallback failed!!!")
                    exit(-1)
                self.set_service_value(0)
            while (1):
                value = self.get_service_value()
                if value == 16 or value == 17 or value == 18:
                    break;
                else:
                    time.sleep(1)
        else:
            print("Unsuccessful login")

    def queryfininfo(self, querytype, params, synchronousflag=False, securityIDSourceAndTypes=[], securityIdList=[]):

        if synchronousflag:  # 同步
            self.queryfininfosynchronous(querytype, params, securityIDSourceAndTypes, securityIdList)
        else:  # 异步
            self.queryfininfoasynchronous(querytype, params, securityIDSourceAndTypes, securityIdList)

    def queryfininfosynchronous(self, querytype, params, securityIDSourceAndTypes=[], securityIdList=[]):

        request = {"DataType": querytype}
        request["Params"] = params
        securitySourceTypes = []
        securityIds = []

        for securityIDSourceAndType in iter(securityIDSourceAndTypes):
            securitySourceTypes.append(securityIDSourceAndType)
        if len(securitySourceTypes) > 0:
            request["MarketdataTypes"] = securitySourceTypes
        for id in securityIdList:
            securityIds.append(id)
        if len(securityIds) > 0:
            request["HTSCSecurityIDs"] = securityIds
        print(request)
        request_info = json.dumps(request)

        rettempdata = mdc_gateway_client.python_request_fin_info_query_sync(request_info, len(request_info))
        retdatalenstr = mdc_gateway_client.cdata(rettempdata, self.min_synchronous_data_len)
        retdatalen = int(retdatalenstr)
        if retdatalen == self.ineffective_size:
            return "invalid request"
        else:
            if self.min_synchronous_data_len < retdatalen < QueryServerConfig.MAX_QUERY_SIZE:
                retdatastr = mdc_gateway_client.cdata(rettempdata, retdatalen)
                result = json.loads(retdatastr[self.min_synchronous_data_len:retdatalen])
                return result
            else:
                return "invalid data"

    def queryfininfoasynchronous(self, querytype, params, securityIDSourceAndTypes=[], securityIdList=[]):

        request = {"DataType": querytype}
        request["Params"] = params
        securitySourceTypes = []
        securityIds = []

        for securityIDSourceAndType in iter(securityIDSourceAndTypes):
            securitySourceTypes.append(securityIDSourceAndType)
        if len(securitySourceTypes) > 0:
            request["MarketdataTypes"] = securitySourceTypes
        for id in securityIdList:
            securityIds.append(id)
        if len(securityIds) > 0:
            request["HTSCSecurityIDs"] = securityIds
        print(request)
        request_info = json.dumps(request)

        ret = mdc_gateway_client.python_request_fin_info_query_async(request_info, len(request_info))
        if ret < 0:
            print("queryfininfo failed!!! reason:%s" % (self.get_error_code_value(ret)))
            exit(-1)

    def queryMdContantCallback(self, securityIDSourceAndTypes, securityIdList, synchronousflag=False):
        self.queryCallback(QueryType.QUERY_CONSTANT, securityIDSourceAndTypes, securityIdList, synchronousflag)

    def queryLastMdContantCallback(self, securityIDSourceAndTypes, securityIdList, synchronousflag=False):
        self.queryCallback(QueryType.QUERY_CONSTANT_TODAY, securityIDSourceAndTypes, securityIdList, synchronousflag)

    def queryETFInfoCallback(self, securityIDSourceAndTypes, securityIdList, synchronousflag=False):
        self.queryCallback(QueryType.QUERY_ETFBASICINFO, securityIDSourceAndTypes, securityIdList, synchronousflag)

    def queryLastMdTickCallback(self, securityIDSourceAndTypes, securityIdList, synchronousflag=False):
        self.queryCallback(QueryType.QUERY_TICK, securityIDSourceAndTypes, securityIdList, synchronousflag)

    # 查询  params:queryType 为int值;securityIdSource 为市场ESecurityIDSource 枚举值;securityType 为 ESecurityIDSource枚举值集合
    def queryCallback(self, queryType, securityIDSourceAndTypes, securityIdList, synchronousflag=False):
        if self.get_login_flag():
            request = {"DataType": queryType}
            securitySourceTypes = []
            securityIds = []

            for securityIDSourceAndType in iter(securityIDSourceAndTypes):
                securitySourceTypes.append(securityIDSourceAndType)
            request["MarketdataTypes"] = securitySourceTypes
            for id in securityIdList:
                securityIds.append(id)
            request["HTSCSecurityIDs"] = securityIds
            print(request)
            request_info = json.dumps(request)
            ret = mdc_gateway_client.python_request_mdquery(request_info, len(request_info))
            if ret < 0:
                print("queryCallback failed!!! reason:%s" % (self.get_error_code_value(ret)))
                exit(-1)
            self.set_query_exit(False)
            while True:
                exit = self.get_query_exit()
                if exit:
                    break;
                else:
                    time.sleep(1)
        else:
            print("Unsuccessful login")

    def login(self, ip, port, username, pwd, istoken, certlog, backuplist,
              queryaddress=QueryServerConfig.QUERY_ADDRESS,
              querycert=QueryServerConfig.QUERY_CERT,
              isSSL=QueryServerConfig.QUERY_IS_SSL,
              thread_count=1):
        self.__init_log__()
        # 必须存在的回调注册，若不存在，则不会产生回调
        mdc_gateway_client.Notify_Regist(notifys)
        loginjson = {}
        loginjson["UserName"] = username
        loginjson["Password"] = pwd
        loginjson["IsToken"] = istoken
        loginjson["MainServerIP"] = ip
        loginjson["MainServerPort"] = port
        loginjson["BackServer"] = backuplist
        loginjson["QueryAddress"] = queryaddress
        loginjson["QueryCert"] = querycert
        loginjson["QuerySSL"] = isSSL
        loginjson["QueryMaxQuerySize"] = QueryServerConfig.MAX_QUERY_SIZE
        loginjson["Thread_Count"] = GateWayServerConfig.THREAD_COUNT
        loginjson["CertPath"] = certlog



        loginstr = json.dumps(loginjson)

        ret = mdc_gateway_client.python_login(loginstr, len(loginstr))
        return ret

    def setCallBack(self, callback):
        recvdata.PyNotify_Regist(callback)

    # def own_deal_log(self,use_init,open_trace = True):
    #     if not use_init:
    #         self.open_trace = open_trace
    #         self.open_file_log = False
    #         self.open_cout_log = False
    #     log = PyLog()
    #     recvdata.PyLog_Regist(log)


class Element():
    def __init__(self, source, type, list):
        self.source = source
        self.type = type
        self.list = list


class SecurityIDSourceAndType():
    def __init__(self, source, type):
        self.source = source
        self.type = type


class SubscribeByIdElement():
    def __init__(self, id, typeList):
        self.id = id
        self.securityIDSourceAndTypeList = typeList


class QueryElement():
    def __init__(self, idList, type):
        self.idList = idList
        self.securityIDSourceAndType = type


class BackupList():
    def __init__(self):
        self.list = mdc_gateway_client.StrIntMap()

    def Add(self, key, value):
        self.list[key.encode(encoding="utf-8")] = value


class StrList():
    def __init__(self):
        self.list = mdc_gateway_client.StrList()

    def Add(self, value):
        self.list.push_back(value.encode(encoding="utf-8"))



#!/usr/bin/python3
import json

from .interface.mdc_gateway_interface import GatewayInterface
from .interface.mdc_gateway_base_define import EMarketDataType, EPlaybackTaskStatus

global interface
interface = GatewayInterface()


def get_interface():
    return interface


class OnRecvMarketData:
    def __init__(self, marketservice):
        self.marketservice = marketservice

    def OnMarketData(self, marketdatajson):
        try:
            if marketdatajson["marketDataType"] == EMarketDataType.MD_TICK:  # .MD_TICK 快照
                if "mdStock" in marketdatajson:  # 股票
                    self.marketservice.onSubscribe_StockType_MD_TICK(marketdatajson["mdStock"])
                elif "mdIndex" in marketdatajson:  # 指数
                    self.marketservice.onSubscribe_IndexType_MD_TICK(marketdatajson["mdIndex"])
                elif "mdBond" in marketdatajson:  # 债券
                    self.marketservice.onSubscribe_BondType_MD_TICK(marketdatajson["mdBond"])
                elif "mdFund" in marketdatajson:  # 基金
                    self.marketservice.onSubscribe_FundType_MD_TICK(marketdatajson["mdFund"])
                elif "mdOption" in marketdatajson:  # 期权
                    self.marketservice.onSubscribe_OptionType_MD_TICK(marketdatajson["mdOption"])
                elif "mdFuture" in marketdatajson:  # 期权
                    self.marketservice.onSubscribe_FuturesType_MD_TICK(marketdatajson["mdFuture"])
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_TRANSACTION:  # .MD_TRANSACTION:逐笔成交
                if "mdTransaction" in marketdatajson:
                    self.marketservice.onSubscribe_MD_TRANSACTION_and_MD_ORDER(marketdatajson)
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_ORDER:  # .MD_ORDER:逐笔委托
                if "mdOrder" in marketdatajson:
                    self.marketservice.onSubscribe_MD_TRANSACTION_and_MD_ORDER(marketdatajson)
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_CONSTANT:  # .MD_CONSTANT:静态信息
                if "mdConstant" in marketdatajson:
                    timestr = json.dumps(marketdatajson["mdConstant"]["HTSCSecurityID"])
                # MD_KLINE:实时数据只提供15S和1MIN K线
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_15S or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_KLINE_1MIN or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_KLINE_5MIN or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_KLINE_15MIN or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_KLINE_30MIN or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_KLINE_60MIN or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_KLINE_1D:
                if "mdKLine" in marketdatajson:
                    self.marketservice.onSubscribe_MD_KLINE(marketdatajson)
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_TWAP_1S or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_TWAP_1MIN:  # .TWAP:TWAP数据
                if "mdTwap" in marketdatajson:
                    timestr = json.dumps(marketdatajson["mdTwap"]["MDTime"])
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_VWAP_1S or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_VWAP_1MIN:  # .VWAP:VWAP数据
                if "mdVwap" in marketdatajson:
                    timestr = json.dumps(marketdatajson["mdVwap"]["MDTime"])
            elif marketdatajson["marketDataType"] == EMarketDataType.AD_FUND_FLOW_ANALYSIS:  # .AD_FUND_FLOW_ANALYSIS:
                if "mdFundFlowAnalysis" in marketdatajson:
                    self.marketservice.onSubscribe_AD_FUND_FLOW_ANALYSIS(marketdatajson["mdFundFlowAnalysis"])
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_ETF_BASICINFO:  # .MD_ETF_BASICINFO:ETF成分股信息
                if "mdETFBasicInfo" in marketdatajson:
                    timestr = json.dumps(marketdatajson["mdETFBasicInfo"]["MDTime"])
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_SECURITY_LENDING:  # .MD_SECURITY_LENDING
                if "mdSecurityLending" in marketdatajson:
                    self.marketservice.onSubscribe_MD_SECURITY_LENDING(marketdatajson["mdSecurityLending"])
        except BaseException as e:
            print("onMarketData error happened!" + marketdatajson)
            print(e)

    def OnPlaybackMarketData(self, marketdatajson):
        # print(marketdatajson)
        try:
            if marketdatajson["marketDataType"] == EMarketDataType.MD_TICK:  # .MD_TICK 快照
                if "mdStock" in marketdatajson:  # 股票
                    self.marketservice.onPlayback_StockType_MD_TICK(marketdatajson["mdStock"])
                elif "mdIndex" in marketdatajson:  # 指数
                    self.marketservice.onPlayback_IndexType_MD_TICK(marketdatajson["mdIndex"])
                elif "mdBond" in marketdatajson:  # 债券
                    self.marketservice.onPlayback_BondType_MD_TICK(marketdatajson["mdBond"])
                elif "mdFund" in marketdatajson:  # 基金
                    self.marketservice.onPlayback_FundType_MD_TICK(marketdatajson["mdFund"])
                elif "mdOption" in marketdatajson:  # 期权
                    self.marketservice.onPlayback_OptionType_MD_TICK(marketdatajson["mdOption"])
                elif "mdFuture" in marketdatajson:  # 期权
                    self.marketservice.onPlayback_FuturesType_MD_TICK(marketdatajson["mdFuture"])
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_TRANSACTION:  # .MD_TRANSACTION:逐笔成交
                if "mdTransaction" in marketdatajson:
                    self.marketservice.onPlayback_MD_TRANSACTION_and_MD_ORDER(marketdatajson)
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_ORDER:  # .MD_ORDER:逐笔委托
                if "mdOrder" in marketdatajson:
                    self.marketservice.onPlayback_MD_TRANSACTION_and_MD_ORDER(marketdatajson)
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_CONSTANT:  # .MD_CONSTANT:静态信息
                if "mdConstant" in marketdatajson:
                    timestr = json.dumps(marketdatajson["mdConstant"]["HTSCSecurityID"])
                # MD_KLINE:实时数据只提供15S和1MIN K线
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_KLINE_15S or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_KLINE_1MIN or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_KLINE_5MIN or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_KLINE_15MIN or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_KLINE_30MIN or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_KLINE_60MIN or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_KLINE_1D:
                if "mdKLine" in marketdatajson:
                    self.marketservice.onPlayback_MD_KLINE(marketdatajson)
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_TWAP_1S or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_TWAP_1MIN:  # .TWAP:TWAP数据
                if "mdTwap" in marketdatajson:
                    timestr = json.dumps(marketdatajson["mdTwap"]["MDTime"])
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_VWAP_1S or marketdatajson[
                "marketDataType"] == EMarketDataType.MD_VWAP_1MIN:  # .VWAP:VWAP数据
                if "mdVwap" in marketdatajson:
                    timestr = json.dumps(marketdatajson["mdVwap"]["MDTime"])
            elif marketdatajson["marketDataType"] == EMarketDataType.AD_FUND_FLOW_ANALYSIS:  # .AD_FUND_FLOW_ANALYSIS:
                if "mdFundFlowAnalysis" in marketdatajson:
                    self.marketservice.onPlayback_AD_FUND_FLOW_ANALYSIS(marketdatajson["mdFundFlowAnalysis"])
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_ETF_BASICINFO:  # .MD_ETF_BASICINFO:ETF成分股信息
                if "mdETFBasicInfo" in marketdatajson:
                    timestr = json.dumps(marketdatajson["mdETFBasicInfo"]["MDTime"])
            elif marketdatajson["marketDataType"] == EMarketDataType.MD_SECURITY_LENDING:  # .MD_SECURITY_LENDING
                if "mdSecurityLending" in marketdatajson:
                    self.marketservice.onPlayback_MD_SECURITY_LENDING(marketdatajson["mdSecurityLending"])
        except BaseException as e:
            print("onMarketData error happened!" + marketdatajson)
            print(e)

    def OnPlaybackPayload(self, playloadstr):
        try:
            interface.set_service_value(4)
            # print(playloadstr)
            playloadjson = json.loads(playloadstr)
            if "taskId" in playloadjson:
                print("Parse Message id:" + playloadjson["taskId"])
            marketDataStream = playloadjson["marketDataStream"]
            if "isFinished" in marketDataStream:
                # print(marketDataStream)
                print("OnPlaybackPayload total number=%d, serial=%d, isfinish=%d" % (
                    marketDataStream["totalNumber"], marketDataStream["serial"], marketDataStream["isFinished"]))
            else:
                print("OnPlaybackPayload total number=%d, serial=%d" % (
                    marketDataStream["totalNumber"], marketDataStream["serial"]))
            marketDataList = marketDataStream["marketDataList"]
            marketDatas = marketDataList["marketDatas"]
            for data in iter(marketDatas):
                self.OnPlaybackMarketData(data)
        except BaseException as e:
            print(e)

    def OnPlaybackStatus(self, statusstr):
        statusresult = "error happened in OnPlaybackStatus"
        try:
            statusjson = json.loads(statusstr)
            statusresult = f'OnPlaybackStatus playback status={statusjson["taskStatus"]}'
            interface.set_service_value(statusjson["taskStatus"])
            if statusjson["taskStatus"] == EPlaybackTaskStatus.CANCELED or statusjson[
                "taskStatus"] == EPlaybackTaskStatus.COMPLETED or statusjson[
                "taskStatus"] == EPlaybackTaskStatus.FAILED:
                interface.mutex.acquire()
                if statusjson["taskId"] in interface.task_id_status:
                    del interface.task_id_status[statusjson["taskId"]]
                interface.mutex.release()
        except BaseException as e:
            print(e)
        self.marketservice.onPlaybackStatus(statusresult)

    def OnPlaybackResponse(self, responsestr):
        responseresult = "error happened in OnPlaybackResponse"
        try:
            responsejson = json.loads(responsestr)
            if "isSuccess" in responsejson:
                if responsejson["isSuccess"]:
                    responseresult = f'OnPlaybackResponse Message id: {responsejson["taskId"]}'
                else:
                    responseresult = f'OnPlaybackResponse failed --> {responsejson["errorContext"]["message"]}'
            else:
                responseresult = f'OnPlaybackResponse :{responsestr}'
        except BaseException as e:
            print(e)
        self.marketservice.onPlaybackResponse(responseresult)

    def OnPlaybackControlResponse(self, responsestr):
        responseresult = "error happened in OnPlaybackControlResponse"
        try:
            responsejson = json.loads(responsestr)
            if "isSuccess" in responsejson:
                if responsejson["isSuccess"]:
                    print(responsejson["currentReplayRate"])
                    responseresult = f'OnPlaybackControlResponse Message id:{responsejson["taskId"]}'
                else:
                    responseresult = f'OnPlaybackControlResponse failed!!! reason ->{responsejson["errorContext"]["message"]}'
            else:
                responseresult = f'OnPlaybackControlResponse :{responsestr}'
        except BaseException as e:
            print(e)
        self.marketservice.onPlaybackControlResponse(responseresult)

    def OnServiceMessage(self, marketDataStr):
        try:
            interface.set_service_value(1)
            marketDataJson = json.loads(marketDataStr)
            self.OnMarketData(marketDataJson)
        except BaseException as e:
            print("error happened in OnServiceMessage")
            print(e)

    def OnSubscribeResponse(self, responsestr):
        try:
            responsejson = json.loads(responsestr)
            issucess = responsejson["isSuccess"]
            if issucess:
                print("Subscribe Success!!!")
            else:
                # print(gateway.getErrorCodeValue(response.errorContext.errorCode))
                print("Subscribe failed!!! reason ->" + (responsestr))
        except BaseException as e:
            print("error happened in OnServiceMessage")
            print(e)

    def OnFinInfoQueryResponse(self, responsestr):
        try:
            responsejson = json.loads(responsestr)
            print(responsejson)
            if "isSuccess" in responsejson:
                if responsejson["isSuccess"]:
                    print("OnFinInfoQueryResponse sucess")
                else:
                    print("OnFinInfoQueryResponse failed!!! reason -> %s" % (responsejson["errorContext"]["message"]))
                    interface.set_query_exit(True)
            else:
                print("OnFinInfoQueryResponse failed!!! reason -> %s" % (responsejson["errorContext"]["message"]))
                interface.set_query_exit(True)
        except BaseException as e:
            print("error happened in OnFinInfoQueryResponse")
            print(e)

    def OnQueryResponse(self, responsestr):
        try:
            responsejson = json.loads(responsestr)
            if "isSuccess" in responsejson:
                if responsejson["isSuccess"]:
                    marketDataStream = responsejson["marketDataStream"]
                    print(
                        "query response total number=%d, serial=%d" % (
                            marketDataStream["totalNumber"], marketDataStream["serial"]))
                    marketDataList = marketDataStream["marketDataList"]
                    marketDatas = marketDataList["marketDatas"]
                    self.marketservice.onQueryResponse(marketDatas)
                    if "isFinished" in marketDataStream:
                        interface.set_query_exit(marketDataStream["isFinished"] == 1)
                else:
                    print("OnQueryResponse failed!!! reason -> %s" % (responsejson["errorContext"]["message"]))
                    interface.set_query_exit(True)
            else:
                print("OnQueryResponse failed!!! reason -> %s" % (responsejson["errorContext"]["message"]))
                interface.set_query_exit(True)
        except BaseException as e:
            print("error happened in OnQueryResponse")
            print(e)

    def OnGeneralError(self, contextstr):
        try:
            contextjson = json.loads(contextstr)
            # print(gateway.getErrorCodeValue(context.errorCode))
            print("OnGeneralError!!! reason -> %s" % (contextjson["message"]))
        except BaseException as e:
            print("error happened in OnGeneralError")
            print(e)

    def OnLoginSuccess(self):
        interface.login_success = True
        print("OnLoginSuccess!!!")

    def OnLoginFailed(self, error_no, message):
        interface.login_success = False
        try:
            print("OnLoginFailed!!! reason -> %s" % message)
        except BaseException as e:
            print("error happened in OnLoginFailed")
            print(e)

    def OnNoConnections(self):
        print("OnNoConnections!!!")
        interface.set_reconnect(True)
        interface.set_no_connections(True)

    def OnReconnect(self):
        print("OnReconnect!!!")
        interface.set_reconnect(True)
        interface.set_reconnect_count(interface.get_reconnect_count() + 1)



from setuptools import find_packages
from setuptools import setup

setup(
    name="insight_gateway_python",
    author="htsc",
    version="4.0.37",
    author_email="insight@htsc.com",
    description="insight_gateway_python",
    long_description="insight_gateway_python",
    license='insightpythonsdk',
    project_urls={
        'Documentation': 'https://packaging.python.org/tutorials/distributing-packages/',
        'Funding': 'https://donate.pypi.org',
        'Source': 'https://github.com/pypa/sampleproject/',
        'Tracker': 'https://github.com/pypa/sampleproject/issues',
    },


    packages= ['insight_gateway_python_v400',
               'insight_gateway_python_v400/com',
               'insight_gateway_python_v400/com/interface',
               'insight_gateway_python_v400/com/libs/python37/x64',
               'insight_gateway_python_v400/com/libs/python36/x64',
               'insight_gateway_python_v400/com/libs/python39/x64',
               'insight_gateway_python_v400/com/cert',
               'insight_gateway_python_v400/com/insight'
               ],
    package_dir={'insight_gateway_python_v400/com/cert': 'insight_gateway_python_v400/com/cert',
                 'insight_gateway_python_v400/com/libs/python37/x64':
                     'insight_gateway_python_v400/com/libs/python37/x64',
                  'insight_gateway_python_v400/com/libs/python36/x64':
                     'insight_gateway_python_v400/com/libs/python36/x64'},
    package_data={'insight_gateway_python_v400/com/cert': ['HTInsightCA.crt', 'InsightClientCert.pem', 'HTISCA.crt', 'InsightClientKeyPkcs8.pem'],
                  'insight_gateway_python_v400/com/libs/python37/x64': ['_mdc_gateway_client.pyd', 'ACE.dll', 'ACE_SSL.dll', 'libeay32.dll', "ssleay32.dll"],
                  'insight_gateway_python_v400/com/libs/python36/x64': ['_mdc_gateway_client.pyd', 'ACE.dll', 'ACE_SSL.dll', 'libeay32.dll', "ssleay32.dll"]},
    install_requires=[],

    python_requires='>=3.4.*',
)


# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _mdc_gateway_client
else:
    import _mdc_gateway_client

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _mdc_gateway_client.delete_SwigPyIterator

    def value(self):
        return _mdc_gateway_client.SwigPyIterator_value(self)

    def incr(self, n=1):
        return _mdc_gateway_client.SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        return _mdc_gateway_client.SwigPyIterator_decr(self, n)

    def distance(self, x):
        return _mdc_gateway_client.SwigPyIterator_distance(self, x)

    def equal(self, x):
        return _mdc_gateway_client.SwigPyIterator_equal(self, x)

    def copy(self):
        return _mdc_gateway_client.SwigPyIterator_copy(self)

    def next(self):
        return _mdc_gateway_client.SwigPyIterator_next(self)

    def __next__(self):
        return _mdc_gateway_client.SwigPyIterator___next__(self)

    def previous(self):
        return _mdc_gateway_client.SwigPyIterator_previous(self)

    def advance(self, n):
        return _mdc_gateway_client.SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        return _mdc_gateway_client.SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        return _mdc_gateway_client.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        return _mdc_gateway_client.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        return _mdc_gateway_client.SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        return _mdc_gateway_client.SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        return _mdc_gateway_client.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _mdc_gateway_client:
_mdc_gateway_client.SwigPyIterator_swigregister(SwigPyIterator)


def cdata(ptr, nelements=1):
    return _mdc_gateway_client.cdata(ptr, nelements)

def memmove(data, indata):
    return _mdc_gateway_client.memmove(data, indata)
class StrIntMap(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def iterator(self):
        return _mdc_gateway_client.StrIntMap_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        return _mdc_gateway_client.StrIntMap___nonzero__(self)

    def __bool__(self):
        return _mdc_gateway_client.StrIntMap___bool__(self)

    def __len__(self):
        return _mdc_gateway_client.StrIntMap___len__(self)
    def __iter__(self):
        return self.key_iterator()
    def iterkeys(self):
        return self.key_iterator()
    def itervalues(self):
        return self.value_iterator()
    def iteritems(self):
        return self.iterator()

    def __getitem__(self, key):
        return _mdc_gateway_client.StrIntMap___getitem__(self, key)

    def __delitem__(self, key):
        return _mdc_gateway_client.StrIntMap___delitem__(self, key)

    def has_key(self, key):
        return _mdc_gateway_client.StrIntMap_has_key(self, key)

    def keys(self):
        return _mdc_gateway_client.StrIntMap_keys(self)

    def values(self):
        return _mdc_gateway_client.StrIntMap_values(self)

    def items(self):
        return _mdc_gateway_client.StrIntMap_items(self)

    def __contains__(self, key):
        return _mdc_gateway_client.StrIntMap___contains__(self, key)

    def key_iterator(self):
        return _mdc_gateway_client.StrIntMap_key_iterator(self)

    def value_iterator(self):
        return _mdc_gateway_client.StrIntMap_value_iterator(self)

    def __setitem__(self, *args):
        return _mdc_gateway_client.StrIntMap___setitem__(self, *args)

    def asdict(self):
        return _mdc_gateway_client.StrIntMap_asdict(self)

    def __init__(self, *args):
        _mdc_gateway_client.StrIntMap_swiginit(self, _mdc_gateway_client.new_StrIntMap(*args))

    def empty(self):
        return _mdc_gateway_client.StrIntMap_empty(self)

    def size(self):
        return _mdc_gateway_client.StrIntMap_size(self)

    def swap(self, v):
        return _mdc_gateway_client.StrIntMap_swap(self, v)

    def begin(self):
        return _mdc_gateway_client.StrIntMap_begin(self)

    def end(self):
        return _mdc_gateway_client.StrIntMap_end(self)

    def rbegin(self):
        return _mdc_gateway_client.StrIntMap_rbegin(self)

    def rend(self):
        return _mdc_gateway_client.StrIntMap_rend(self)

    def clear(self):
        return _mdc_gateway_client.StrIntMap_clear(self)

    def get_allocator(self):
        return _mdc_gateway_client.StrIntMap_get_allocator(self)

    def count(self, x):
        return _mdc_gateway_client.StrIntMap_count(self, x)

    def erase(self, *args):
        return _mdc_gateway_client.StrIntMap_erase(self, *args)

    def find(self, x):
        return _mdc_gateway_client.StrIntMap_find(self, x)

    def lower_bound(self, x):
        return _mdc_gateway_client.StrIntMap_lower_bound(self, x)

    def upper_bound(self, x):
        return _mdc_gateway_client.StrIntMap_upper_bound(self, x)
    __swig_destroy__ = _mdc_gateway_client.delete_StrIntMap

# Register StrIntMap in _mdc_gateway_client:
_mdc_gateway_client.StrIntMap_swigregister(StrIntMap)

class Notify(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        if self.__class__ == Notify:
            _self = None
        else:
            _self = self
        _mdc_gateway_client.Notify_swiginit(self, _mdc_gateway_client.new_Notify(_self, ))
    __swig_destroy__ = _mdc_gateway_client.delete_Notify

    def OnMarketData(self, buf, size):
        return _mdc_gateway_client.Notify_OnMarketData(self, buf, size)

    def OnPlaybackPayload(self, buf, size):
        return _mdc_gateway_client.Notify_OnPlaybackPayload(self, buf, size)

    def OnPlaybackStatus(self, buf, size):
        return _mdc_gateway_client.Notify_OnPlaybackStatus(self, buf, size)

    def OnPlaybackResponse(self, buf, size):
        return _mdc_gateway_client.Notify_OnPlaybackResponse(self, buf, size)

    def OnPlaybackControlResponse(self, buf, size):
        return _mdc_gateway_client.Notify_OnPlaybackControlResponse(self, buf, size)

    def OnServiceMessage(self, buf, size):
        return _mdc_gateway_client.Notify_OnServiceMessage(self, buf, size)

    def OnSubscribeResponse(self, buf, size):
        return _mdc_gateway_client.Notify_OnSubscribeResponse(self, buf, size)

    def OnQueryResponse(self, buf, size):
        return _mdc_gateway_client.Notify_OnQueryResponse(self, buf, size)

    def OnGeneralError(self, buf, size):
        return _mdc_gateway_client.Notify_OnGeneralError(self, buf, size)

    def OnFinInfoQueryResponse(self, buf, size):
        return _mdc_gateway_client.Notify_OnFinInfoQueryResponse(self, buf, size)

    def OnLoginSuccess(self):
        return _mdc_gateway_client.Notify_OnLoginSuccess(self)

    def OnLoginFailed(self, error_no, message):
        return _mdc_gateway_client.Notify_OnLoginFailed(self, error_no, message)

    def OnNoConnections(self):
        return _mdc_gateway_client.Notify_OnNoConnections(self)

    def OnReconnect(self):
        return _mdc_gateway_client.Notify_OnReconnect(self)
    def __disown__(self):
        self.this.disown()
        _mdc_gateway_client.disown_Notify(self)
        return weakref.proxy(self)

# Register Notify in _mdc_gateway_client:
_mdc_gateway_client.Notify_swigregister(Notify)


def Notify_Regist(notify):
    return _mdc_gateway_client.Notify_Regist(notify)

def Log_Regist(log):
    return _mdc_gateway_client.Log_Regist(log)

def python_init_env():
    return _mdc_gateway_client.python_init_env()

def python_fini_env():
    return _mdc_gateway_client.python_fini_env()

def python_open_file_log():
    return _mdc_gateway_client.python_open_file_log()

def python_close_file_log():
    return _mdc_gateway_client.python_close_file_log()

def python_open_cout_log():
    return _mdc_gateway_client.python_open_cout_log()

def python_close_cout_log():
    return _mdc_gateway_client.python_close_cout_log()

def python_open_trace():
    return _mdc_gateway_client.python_open_trace()

def python_close_trace():
    return _mdc_gateway_client.python_close_trace()

def python_is_trace():
    return _mdc_gateway_client.python_is_trace()

def python_open_compress():
    return _mdc_gateway_client.python_open_compress()

def python_close_compress():
    return _mdc_gateway_client.python_close_compress()

def python_is_compress():
    return _mdc_gateway_client.python_is_compress()

def python_open_response_callback():
    return _mdc_gateway_client.python_open_response_callback()

def python_close_response_callback():
    return _mdc_gateway_client.python_close_response_callback()

def python_is_response_callback():
    return _mdc_gateway_client.python_is_response_callback()

def python_debug_print(msg):
    return _mdc_gateway_client.python_debug_print(msg)

def python_warning_print(msg):
    return _mdc_gateway_client.python_warning_print(msg)

def python_error_print(msg):
    return _mdc_gateway_client.python_error_print(msg)

def python_get_version():
    return _mdc_gateway_client.python_get_version()

def python_get_error_code_value(code):
    return _mdc_gateway_client.python_get_error_code_value(code)

def python_get_task_id():
    return _mdc_gateway_client.python_get_task_id()

def python_login(loginstr, size):
    return _mdc_gateway_client.python_login(loginstr, size)

def python_logout():
    return _mdc_gateway_client.python_logout()

def python_subscribe(buf, size):
    return _mdc_gateway_client.python_subscribe(buf, size)

def python_request_mdquery(buf, size):
    return _mdc_gateway_client.python_request_mdquery(buf, size)

def python_request_fin_info_query_sync(buf, size):
    return _mdc_gateway_client.python_request_fin_info_query_sync(buf, size)

def python_request_fin_info_query_async(buf, size):
    return _mdc_gateway_client.python_request_fin_info_query_async(buf, size)

def python_request_playback(buf, size):
    return _mdc_gateway_client.python_request_playback(buf, size)





#!/usr/bin/python3
# -*- coding: utf-8 -*-
from interface.mdc_gateway_head import mdc_gateway_client

class PyLog(mdc_gateway_client.PyLog):
    def __init__(self):
        mdc_gateway_client.PyLog.__init__(self)

    def log(self,line):
        # line中已包含换行符,end='' 替换换行符
        if not str(line).__contains__("heartbeat"):
            print(line.decode(),end='')


from .. import data_handle


def query_fin_info(query_type, params):
    return data_handle.get_interface().queryfininfosynchronous(query_type, params)


def query_basicInfo_by_type(dictmarketdatatypes, isToday=True):
    security_id_list = []  # 置空表示不额外查询某些标的
    if isToday:
        query_last_mdcontant(dictmarketdatatypes, security_id_list)
    else:
        query_mdcontant(dictmarketdatatypes, security_id_list)


def query_basicInfo_by_id(listsecurityid, isToday=True):
    listmarketdatatype = []  # 置空表示不额外查询某些标的
    if isToday:
        query_last_mdcontant(listmarketdatatype, listsecurityid)
    else:
        query_mdcontant(listmarketdatatype, listsecurityid)


# 查询历史上所有的指定证券的基础信息 -- 在data_handle.py 数据回调接口OnMarketData()中marketdata.marketDataType = MD_CONSTANT
# params:securityIdSource 为市场ESecurityIDSource 枚举值;securityType 为 ESecurityType枚举值
def query_mdcontant(security_idsource_and_types, security_id_list):
    data_handle.get_interface().queryMdContantCallback(security_idsource_and_types, security_id_list)


# 查询今日最新的指定证券的基础信息 -- 在data_handle.py 数据回调接口OnMarketData()中marketdata.marketDataType = MD_CONSTANT
# params:securityIdSource 为市场ESecurityIDSource 枚举值;securityType 为 ESecurityType枚举值
def query_last_mdcontant(security_idsource_and_types, security_id_list):
    # 按市场查询
    # 沪市 股票
    data_handle.get_interface().queryLastMdContantCallback(security_idsource_and_types, security_id_list)


# 查询指定证券的ETF的基础信息 -- 在data_handle.py 数据回调接口OnMarketData()中marketdata.marketDataType = MD_ETF_BASICINFO
# params:securityIdSource 为市场ESecurityIDSource 枚举值;securityType 为 ESecurityType枚举值
def query_ETFinfo(securityIDSource, securityType):  # 查询指定证券的ETF的基础信息
    # params:securityIDSource 为 ESecurityIDSource枚举值
    # params:securityType 为 ESecurityType枚举值
    security_idsource_and_types = []
    # 沪市 股票
    idsource_and_type = {"ESecurityIDSource": securityIDSource, "ESecurityType": securityType}
    security_idsource_and_types.append(idsource_and_type)

    # securityIDSourceAndTypes 与 securityIdList并集
    security_id_list = []  # 置空表示不额外查询某些标的
    # params:security_id_list 为 标的集合
    data_handle.get_interface().queryETFInfoCallback(security_idsource_and_types, security_id_list)


# 查询指定证券的最新一条Tick数据 -- 在data_handle.py 数据回调接口OnMarketData()中marketdata.marketDataType = MD_TICK
# params:securityIdSource 为市场ESecurityIDSource 枚举值;securityType 为 ESecurityType枚举值
def query_last_tick_by_type(securityIDSource, securityType):  # 通过证券类型和数据源查询最新Tick数据
    # params:security_idsource 为 ESecurityIDSource枚举值
    # params:security_type 为 ESecurityType枚举值
    # 沪市 股票

    security_idsource_and_types = []
    idsource_and_type = {"ESecurityIDSource": securityIDSource, "ESecurityType": securityType}
    security_idsource_and_types.append(idsource_and_type)

    # security_id_list 标的 列表
    security_id_list = []  # 置空表示不额外查询某些标的
    # security_idsource_and_types 与 security_id_list 并集
    data_handle.get_interface().queryLastMdTickCallback(security_idsource_and_types, security_id_list)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from .. import data_handle
from ..interface.mdc_gateway_base_define import ESubscribeActionType


# 根据证券数据来源订阅行情数据,由三部分确定行情数据
# 行情源(SecurityIdSource):XSHG(沪市)|XSHE(深市)|...
# 证券类型(SecurityType):BondType(债)|StockType(股)|FundType(基)|IndexType(指)|OptionType(期权)|...
# 数据类型(MarketDataTypes):MD_TICK(快照)|MD_TRANSACTION(逐笔成交)|MD_ORDER(逐笔委托)|...
def subscribe_by_type(dictmarketdatatypes, MDSubscribe=ESubscribeActionType.COVERAGE):
    # element
    # params1: ESecurityIDSource枚举值 --行情源
    # params2: ESecurityType的枚举值 --证券类型
    # params3: EMarketDataType的枚举值集合 --数据类型
    listmarketdatatype = []
    for dictmarketdatatype in iter(dictmarketdatatypes):
        listmarketdatatype.append(dictmarketdatatype)
    data_handle.get_interface().subscribebytype(MDSubscribe, listmarketdatatype)
    sync()


# 根据证券ID来源订阅行情数据
def subscribe_by_id(dictHTSCSecurityIDs, MDSubscribe=ESubscribeActionType.COVERAGE):
    listHTSCSecurityIDs = []
    for dictHTSCSecurityID in iter(dictHTSCSecurityIDs):
        listHTSCSecurityIDs.append(dictHTSCSecurityID)
    data_handle.get_interface().subscribebyid(MDSubscribe, listHTSCSecurityIDs)
    sync()


# 阻塞当前线程，防止本模块执行退出操作
def sync():
    print("input any key to exit >>>")
    line = input()
    if len(str(line)) > 0:
        print("sync: input-->>" + str(line) + ",then exit this sync.")



import time
from datetime import datetime
from .. import data_handle


# 回放接口 (注意：securitylist 和 securityIdList取并集!!!)
# 回放限制
# 对于回放而言，时间限制由股票只数和天数的乘积决定，要求 回放只数 × 回放天数 × 证券权重 ≤ 450，交易时间段内回放功能 乘积<=200。
# Tick/Transaction/Order回放时间范围限制是30天，每支证券权重为1，即可以回放15只股票30天以内的数据或450支股票1天内数据。
# 日K数据回放时间范围限制是365天，每支证券权重为0.005。
# 分钟K线数据回放时间范围限制是90天，每支证券权重0.05。
# 数据最早可以回放到 2017年1月2日
def playback(htscsecurityID_and_types, exrights_type, start_time, stop_time):
    # 回放数据类型 EMarketDataType 详情请参阅 数据手册EMarketDataType
    # 示例：MD_TICK
    if isVaildDate(start_time) and isVaildDate(stop_time):
        data_handle.get_interface().playCallback(htscsecurityID_and_types, exrights_type, start_time, stop_time)
    else:
        print("Illegal time parameter")


def isVaildDate(playdate):
    if len(playdate) == 14:
        try:
            time.strptime(playdate, "%Y%m%d%H%M%S")
            return True
        except BaseException as e:
            return False
    return False


# 盘中回放接口 --securitylist 和 securityIdList取并集
# Can only query data for one day
# 是否按照mdtime排序
def play_back_oneday(htscsecurityID_and_types, exrights_type, isMdtime=True):
    # 回放数据类型 EMarketDataType 详情请参阅 数据手册EMarketDataType
    # 示例：MD_TICK
    cur_time = datetime.now()

    date_str = cur_time.strftime('%Y%m%d')
    print(date_str)

    start_time = f"{date_str}000000"
    stop_time = f"{date_str}235959"

    # 特别注意！！！！
    # security_id_list 注意回放限制
    print(htscsecurityID_and_types)
    data_handle.get_interface().playCallback(htscsecurityID_and_types, exrights_type, start_time, stop_time,
                                                 isMdtime)
